# CCSDS-NDM: CCSDS Navigation Data Messages Read/Write Library
#
# Copyright (C) Egemen Imre
#
# Licensed under GNU GPL v3.0. See LICENSE.rst for more info.
"""
CCSDS Navigation Data Messages Mappings

"""

from enum import Enum, auto

from ccsds_ndm.models import ndmxml1, ndmxml2, ndmxml4


class NDMFileFormats(Enum):
    """
    NDM file formats.
    """

    XML = auto()
    KVN = auto()
    JSON = auto()


class _NdmDataType(Enum):
    """
    Enumeration of NDM (Navigation Data Message) data types and their associated metadata.

    This enum maps NDM data types (such as OEM, AEM, APM, etc.) to their corresponding
    XML schema classes, versions, and schema versions. It provides a centralized registry
    for parsing and rendering NDM XML data according to CCSDS NDM XML schemas.

    Parameters
    ----------
    ndm_id : str
        NDM data identifier in lowercase (e.g., 'aem', 'oem', 'ndm')
    clazz : type
        Class corresponding to the NDM data type, used for XML parsing and rendering
        Version string of the NDM data type (e.g., '1.0', '2.0', '3.0', '4.0')
    req_combi_version : int
        Required NDM XML schema version required for this NDM data type
    is_combi : bool
        Flag indicating whether this NDM type can contain multiple data types
        (True for combined NDM containers, False for individual data types)

    Notes
    -----
    - Each enum member represents a specific NDM data type and version combination
    - The class attribute references generated xsdata classes from ndmxml1, ndmxml2, and ndmxml4
    """

    ACMv2 = (ndmxml4.Acm.Meta.name, ndmxml4.Acm, "2.0", 4, False)

    AEMv1 = (ndmxml2.Aem.Meta.name, ndmxml2.Aem, "1.0", 2, False)
    AEMv2 = (ndmxml4.Aem.Meta.name, ndmxml4.Aem, "2.0", 4, False)

    APMv1 = (ndmxml2.Apm.Meta.name, ndmxml2.Apm, "1.0", 2, False)
    APMv2 = (ndmxml4.Apm.Meta.name, ndmxml4.Apm, "2.0", 4, False)

    CDMv1 = (ndmxml4.Cdm.Meta.name, ndmxml4.Cdm, "1.0", 4, False)

    OCMv3 = (ndmxml4.Ocm.Meta.name, ndmxml4.Ocm, "3.0", 4, False)

    OEMv2 = (ndmxml2.Oem.Meta.name, ndmxml2.Oem, "2.0", 2, False)
    OEMv3 = (ndmxml4.Oem.Meta.name, ndmxml4.Oem, "3.0", 4, False)

    OMMv2 = (ndmxml2.Omm.Meta.name, ndmxml2.Omm, "2.0", 2, False)
    OMMv3 = (ndmxml4.Omm.Meta.name, ndmxml4.Omm, "3.0", 4, False)

    OPMv1 = (ndmxml1.Opm.Meta.name, ndmxml1.Opm, "1.0", 2, False)
    OPMv2 = (ndmxml2.Opm.Meta.name, ndmxml2.Opm, "2.0", 2, False)
    OPMv3 = (ndmxml4.Opm.Meta.name, ndmxml4.Opm, "3.0", 4, False)

    RDMv1 = (ndmxml4.Rdm.Meta.name, ndmxml4.Rdm, "1.0", 4, False)

    TDMv1 = (ndmxml1.Tdm.Meta.name, ndmxml1.Tdm, "1.0", 2, False)
    TDMv2 = (ndmxml4.Tdm.Meta.name, ndmxml4.Tdm, "2.0", 4, False)

    # The following can contain multiple NDM Data Types
    # (e.g. OMM, APM etc.) and are identified as "NDM" in the XML tag.
    NDMv2 = (ndmxml2.Ndm.Meta.name, ndmxml2.Ndm, "2.0", 2, True)
    NDMv4 = (ndmxml4.Ndm.Meta.name, ndmxml4.Ndm, "4.0", 4, True)

    def __init__(
        self, ndm_id: str, clazz, version: str, req_combi_version: int, is_combi
    ):
        self.clazz = clazz
        self.ndm_id = ndm_id.strip()
        self.version = version.strip()
        self.req_combi_version = req_combi_version
        self.is_combi = is_combi

    @staticmethod
    def find_ndm_type_by_id(ndm_id, version) -> "_NdmDataType":
        """
        Finds the NDM Data Type corresponding to the requested id.

        Parameters
        ----------
        ndm_id : str
            NDM data id (e.g. `aem` or `ndm`)
        version : str
            Version of the NDM data type
        Returns
        -------
        ndm_data_type
            correct `_NdmDataType` enum corresponding to the id
        """
        for ndm_data in _NdmDataType:
            if ndm_data.ndm_id == ndm_id and ndm_data.version == version:
                return ndm_data

        raise ValueError(
            f"No NDM data type found for id '{ndm_id}' and version '{version}'"
        )

    @staticmethod
    def find_ndm_type_by_class_id(ndm_class_id, version) -> "_NdmDataType":
        """
        Finds the NDM Data Type corresponding to the requested id.

        Parameters
        ----------
        ndm_class_id : str
            NDM class id (e.g. `CCSDS_ACM_VERS` or `CCSDS_OEM_VERS`)
        version : str
            Version of the NDM data type
        Returns
        -------
        ndm_data_type
            correct `_NdmDataType` enum corresponding to the id
        """
        for ndm_data in _NdmDataType:
            if ndm_data.clazz.id == ndm_class_id and ndm_data.version == version:
                return ndm_data

        raise ValueError(
            f"No NDM data type found for id '{ndm_class_id}' and version '{version}'"
        )

    @staticmethod
    def find_combi_version(ndm_type: "_NdmDataType") -> "_NdmDataType":
        """
        Finds the appropriate combined NDM version for a given NDM data type.

        Parameters
        ----------
        ndm_type : _NdmDataType
            The NDM data type for which to find the combined version

        Returns
        -------
        _NdmDataType
            The appropriate combined NDM version that supports the given NDM data type
        """
        for ndm_data in _NdmDataType:
            if (
                ndm_data.is_combi
                and ndm_data.req_combi_version >= ndm_type.req_combi_version
            ):
                return ndm_data

        raise ValueError(
            f"No combined NDM version found that supports the required version for {ndm_type}"
        )



"""This file was generated by xsdata, v26.1, on 2026-02-18 00:40:52

Generator: DataclassGenerator
See: https://xsdata.readthedocs.io/
"""

from __future__ import annotations

from dataclasses import dataclass, field
from enum import Enum

from ccsds_ndm.models.ndmxml1.ndmxml_1_0_navwg_common import (
    AreaType,
    LengthType,
    MassType,
    Ms2Type,
    ObjectDescriptionType,
    OdParametersType,
    PositionType,
    PositionUnits,
    VelocityType,
    VelocityUnits,
    YesNoType,
)

__NAMESPACE__ = "urn:ccsds:recommendation:navigation:schema:ndmxml"


@dataclass(kw_only=True)
class CdmHeader:
    class Meta:
        name = "cdmHeader"

    comment: list[str] = field(
        default_factory=list,
        metadata={
            "name": "COMMENT",
            "type": "Element",
            "namespace": "",
        },
    )
    creation_date: str = field(
        metadata={
            "name": "CREATION_DATE",
            "type": "Element",
            "namespace": "",
            "required": True,
            "pattern": r"\-?\d{4}\d*-((\d{2}\-\d{2})|\d{3})T\d{2}:\d{2}:\d{2}(\.\d*)?(Z|[+|\-]\d{2}:\d{2})?|[+|\-]?\d*(\.\d*)?",
        }
    )
    originator: str = field(
        metadata={
            "name": "ORIGINATOR",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    message_for: None | str = field(
        default=None,
        metadata={
            "name": "MESSAGE_FOR",
            "type": "Element",
            "namespace": "",
        },
    )
    message_id: str = field(
        metadata={
            "name": "MESSAGE_ID",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )


class CovarianceMethodType(Enum):
    CALCULATED = "CALCULATED"
    CALCULATED_1 = "calculated"
    DEFAULT = "DEFAULT"
    DEFAULT_1 = "default"


class DvUnits(Enum):
    M_S = "m/s"


class M2Units(Enum):
    M_2 = "m**2"


class M2KgUnits(Enum):
    M_2_KG = "m**2/kg"


class M2S2Units(Enum):
    M_2_S_2 = "m**2/s**2"


class M2S3Units(Enum):
    M_2_S_3 = "m**2/s**3"


class M2S4Units(Enum):
    M_2_S_4 = "m**2/s**4"


class M2SUnits(Enum):
    M_2_S = "m**2/s"


class M3KgUnits(Enum):
    M_3_KG = "m**3/kg"


class M3Kgs2Units(Enum):
    M_3_KG_S_2 = "m**3/(kg*s**2)"


class M3KgsUnits(Enum):
    M_3_KG_S = "m**3/(kg*s)"


class M4Kg2Units(Enum):
    M_4_KG_2 = "m**4/kg**2"


class ManeuverableType(Enum):
    YES = "YES"
    YES_1 = "yes"
    NO = "NO"
    NO_1 = "no"
    N_A = "N/A"
    N_A_1 = "n/a"


class ObjectType(Enum):
    OBJECT1 = "OBJECT1"
    OBJECT1_1 = "object1"
    OBJECT2 = "OBJECT2"
    OBJECT2_1 = "object2"


class ReferenceFrameType(Enum):
    EME2000 = "EME2000"
    EME2000_1 = "eme2000"
    GCRF = "GCRF"
    GCRF_1 = "gcrf"
    ITRF = "ITRF"
    ITRF_1 = "itrf"


class ScreenVolumeFrameType(Enum):
    RTN = "RTN"
    RTN_1 = "rtn"
    TVN = "TVN"
    TVN_1 = "tvn"


class ScreenVolumeShapeType(Enum):
    ELLIPSOID = "ELLIPSOID"
    ELLIPSOID_1 = "ellipsoid"
    BOX = "BOX"
    BOX_1 = "box"


class WkgUnits(Enum):
    W_KG = "W/kg"


@dataclass(kw_only=True)
class CdmMetadata:
    class Meta:
        name = "cdmMetadata"

    comment: list[str] = field(
        default_factory=list,
        metadata={
            "name": "COMMENT",
            "type": "Element",
            "namespace": "",
        },
    )
    object_value: ObjectType = field(
        metadata={
            "name": "OBJECT",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    object_designator: str = field(
        metadata={
            "name": "OBJECT_DESIGNATOR",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    catalog_name: str = field(
        metadata={
            "name": "CATALOG_NAME",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    object_name: str = field(
        metadata={
            "name": "OBJECT_NAME",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    international_designator: str = field(
        metadata={
            "name": "INTERNATIONAL_DESIGNATOR",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    object_type: None | ObjectDescriptionType = field(
        default=None,
        metadata={
            "name": "OBJECT_TYPE",
            "type": "Element",
            "namespace": "",
        },
    )
    operator_contact_position: None | str = field(
        default=None,
        metadata={
            "name": "OPERATOR_CONTACT_POSITION",
            "type": "Element",
            "namespace": "",
        },
    )
    operator_organization: None | str = field(
        default=None,
        metadata={
            "name": "OPERATOR_ORGANIZATION",
            "type": "Element",
            "namespace": "",
        },
    )
    operator_phone: None | str = field(
        default=None,
        metadata={
            "name": "OPERATOR_PHONE",
            "type": "Element",
            "namespace": "",
        },
    )
    operator_email: None | str = field(
        default=None,
        metadata={
            "name": "OPERATOR_EMAIL",
            "type": "Element",
            "namespace": "",
        },
    )
    ephemeris_name: str = field(
        metadata={
            "name": "EPHEMERIS_NAME",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    covariance_method: CovarianceMethodType = field(
        metadata={
            "name": "COVARIANCE_METHOD",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    maneuverable: ManeuverableType = field(
        metadata={
            "name": "MANEUVERABLE",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    orbit_center: None | str = field(
        default=None,
        metadata={
            "name": "ORBIT_CENTER",
            "type": "Element",
            "namespace": "",
        },
    )
    ref_frame: ReferenceFrameType = field(
        metadata={
            "name": "REF_FRAME",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    gravity_model: None | str = field(
        default=None,
        metadata={
            "name": "GRAVITY_MODEL",
            "type": "Element",
            "namespace": "",
        },
    )
    atmospheric_model: None | str = field(
        default=None,
        metadata={
            "name": "ATMOSPHERIC_MODEL",
            "type": "Element",
            "namespace": "",
        },
    )
    n_body_perturbations: None | str = field(
        default=None,
        metadata={
            "name": "N_BODY_PERTURBATIONS",
            "type": "Element",
            "namespace": "",
        },
    )
    solar_rad_pressure: None | YesNoType = field(
        default=None,
        metadata={
            "name": "SOLAR_RAD_PRESSURE",
            "type": "Element",
            "namespace": "",
        },
    )
    earth_tides: None | YesNoType = field(
        default=None,
        metadata={
            "name": "EARTH_TIDES",
            "type": "Element",
            "namespace": "",
        },
    )
    intrack_thrust: None | YesNoType = field(
        default=None,
        metadata={
            "name": "INTRACK_THRUST",
            "type": "Element",
            "namespace": "",
        },
    )


@dataclass(kw_only=True)
class CdmPositionType(PositionType):
    class Meta:
        name = "cdmPositionType"

    units: PositionUnits = field(
        metadata={
            "type": "Attribute",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class CdmVelocityType(VelocityType):
    class Meta:
        name = "cdmVelocityType"

    units: VelocityUnits = field(
        metadata={
            "type": "Attribute",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class DvType:
    class Meta:
        name = "dvType"

    value: float = field(
        metadata={
            "required": True,
        }
    )
    units: DvUnits = field(
        metadata={
            "type": "Attribute",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class M2Type:
    class Meta:
        name = "m2Type"

    value: float = field(
        metadata={
            "required": True,
        }
    )
    units: M2Units = field(
        metadata={
            "type": "Attribute",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class M2KgType:
    class Meta:
        name = "m2kgType"

    value: float = field(
        metadata={
            "required": True,
            "min_inclusive": 0.0,
        }
    )
    units: M2KgUnits = field(
        metadata={
            "type": "Attribute",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class M2S2Type:
    class Meta:
        name = "m2s2Type"

    value: float = field(
        metadata={
            "required": True,
        }
    )
    units: M2S2Units = field(
        metadata={
            "type": "Attribute",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class M2S3Type:
    class Meta:
        name = "m2s3Type"

    value: float = field(
        metadata={
            "required": True,
        }
    )
    units: M2S3Units = field(
        metadata={
            "type": "Attribute",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class M2S4Type:
    class Meta:
        name = "m2s4Type"

    value: float = field(
        metadata={
            "required": True,
        }
    )
    units: M2S4Units = field(
        metadata={
            "type": "Attribute",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class M2SType:
    class Meta:
        name = "m2sType"

    value: float = field(
        metadata={
            "required": True,
        }
    )
    units: M2SUnits = field(
        metadata={
            "type": "Attribute",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class M3KgType:
    class Meta:
        name = "m3kgType"

    value: float = field(
        metadata={
            "required": True,
        }
    )
    units: M3KgUnits = field(
        metadata={
            "type": "Attribute",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class M3Kgs2Type:
    class Meta:
        name = "m3kgs2Type"

    value: float = field(
        metadata={
            "required": True,
        }
    )
    units: M3Kgs2Units = field(
        metadata={
            "type": "Attribute",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class M3KgsType:
    class Meta:
        name = "m3kgsType"

    value: float = field(
        metadata={
            "required": True,
        }
    )
    units: M3KgsUnits = field(
        metadata={
            "type": "Attribute",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class M4Kg2Type:
    class Meta:
        name = "m4kg2Type"

    value: float = field(
        metadata={
            "required": True,
        }
    )
    units: M4Kg2Units = field(
        metadata={
            "type": "Attribute",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class WkgType:
    class Meta:
        name = "wkgType"

    value: float = field(
        metadata={
            "required": True,
            "min_inclusive": 0.0,
        }
    )
    units: WkgUnits = field(
        metadata={
            "type": "Attribute",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class AdditionalParametersType:
    class Meta:
        name = "additionalParametersType"

    comment: list[str] = field(
        default_factory=list,
        metadata={
            "name": "COMMENT",
            "type": "Element",
            "namespace": "",
        },
    )
    area_pc: None | AreaType = field(
        default=None,
        metadata={
            "name": "AREA_PC",
            "type": "Element",
            "namespace": "",
        },
    )
    area_drg: None | AreaType = field(
        default=None,
        metadata={
            "name": "AREA_DRG",
            "type": "Element",
            "namespace": "",
        },
    )
    area_srp: None | AreaType = field(
        default=None,
        metadata={
            "name": "AREA_SRP",
            "type": "Element",
            "namespace": "",
        },
    )
    mass: None | MassType = field(
        default=None,
        metadata={
            "name": "MASS",
            "type": "Element",
            "namespace": "",
        },
    )
    cd_area_over_mass: None | M2KgType = field(
        default=None,
        metadata={
            "name": "CD_AREA_OVER_MASS",
            "type": "Element",
            "namespace": "",
        },
    )
    cr_area_over_mass: None | M2KgType = field(
        default=None,
        metadata={
            "name": "CR_AREA_OVER_MASS",
            "type": "Element",
            "namespace": "",
        },
    )
    thrust_acceleration: None | Ms2Type = field(
        default=None,
        metadata={
            "name": "THRUST_ACCELERATION",
            "type": "Element",
            "namespace": "",
        },
    )
    sedr: None | WkgType = field(
        default=None,
        metadata={
            "name": "SEDR",
            "type": "Element",
            "namespace": "",
        },
    )


@dataclass(kw_only=True)
class CdmCovarianceType:
    class Meta:
        name = "cdmCovarianceType"

    comment: list[str] = field(
        default_factory=list,
        metadata={
            "name": "COMMENT",
            "type": "Element",
            "namespace": "",
        },
    )
    cr_r: M2Type = field(
        metadata={
            "name": "CR_R",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    ct_r: M2Type = field(
        metadata={
            "name": "CT_R",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    ct_t: M2Type = field(
        metadata={
            "name": "CT_T",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    cn_r: M2Type = field(
        metadata={
            "name": "CN_R",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    cn_t: M2Type = field(
        metadata={
            "name": "CN_T",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    cn_n: M2Type = field(
        metadata={
            "name": "CN_N",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    crdot_r: M2SType = field(
        metadata={
            "name": "CRDOT_R",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    crdot_t: M2SType = field(
        metadata={
            "name": "CRDOT_T",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    crdot_n: M2SType = field(
        metadata={
            "name": "CRDOT_N",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    crdot_rdot: M2S2Type = field(
        metadata={
            "name": "CRDOT_RDOT",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    ctdot_r: M2SType = field(
        metadata={
            "name": "CTDOT_R",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    ctdot_t: M2SType = field(
        metadata={
            "name": "CTDOT_T",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    ctdot_n: M2SType = field(
        metadata={
            "name": "CTDOT_N",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    ctdot_rdot: M2S2Type = field(
        metadata={
            "name": "CTDOT_RDOT",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    ctdot_tdot: M2S2Type = field(
        metadata={
            "name": "CTDOT_TDOT",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    cndot_r: M2SType = field(
        metadata={
            "name": "CNDOT_R",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    cndot_t: M2SType = field(
        metadata={
            "name": "CNDOT_T",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    cndot_n: M2SType = field(
        metadata={
            "name": "CNDOT_N",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    cndot_rdot: M2S2Type = field(
        metadata={
            "name": "CNDOT_RDOT",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    cndot_tdot: M2S2Type = field(
        metadata={
            "name": "CNDOT_TDOT",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    cndot_ndot: M2S2Type = field(
        metadata={
            "name": "CNDOT_NDOT",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    cdrg_r: None | M3KgType = field(
        default=None,
        metadata={
            "name": "CDRG_R",
            "type": "Element",
            "namespace": "",
        },
    )
    cdrg_t: None | M3KgType = field(
        default=None,
        metadata={
            "name": "CDRG_T",
            "type": "Element",
            "namespace": "",
        },
    )
    cdrg_n: None | M3KgType = field(
        default=None,
        metadata={
            "name": "CDRG_N",
            "type": "Element",
            "namespace": "",
        },
    )
    cdrg_rdot: None | M3KgsType = field(
        default=None,
        metadata={
            "name": "CDRG_RDOT",
            "type": "Element",
            "namespace": "",
        },
    )
    cdrg_tdot: None | M3KgsType = field(
        default=None,
        metadata={
            "name": "CDRG_TDOT",
            "type": "Element",
            "namespace": "",
        },
    )
    cdrg_ndot: None | M3KgsType = field(
        default=None,
        metadata={
            "name": "CDRG_NDOT",
            "type": "Element",
            "namespace": "",
        },
    )
    cdrg_drg: None | M4Kg2Type = field(
        default=None,
        metadata={
            "name": "CDRG_DRG",
            "type": "Element",
            "namespace": "",
        },
    )
    csrp_r: None | M3KgType = field(
        default=None,
        metadata={
            "name": "CSRP_R",
            "type": "Element",
            "namespace": "",
        },
    )
    csrp_t: None | M3KgType = field(
        default=None,
        metadata={
            "name": "CSRP_T",
            "type": "Element",
            "namespace": "",
        },
    )
    csrp_n: None | M3KgType = field(
        default=None,
        metadata={
            "name": "CSRP_N",
            "type": "Element",
            "namespace": "",
        },
    )
    csrp_rdot: None | M3KgsType = field(
        default=None,
        metadata={
            "name": "CSRP_RDOT",
            "type": "Element",
            "namespace": "",
        },
    )
    csrp_tdot: None | M3KgsType = field(
        default=None,
        metadata={
            "name": "CSRP_TDOT",
            "type": "Element",
            "namespace": "",
        },
    )
    csrp_ndot: None | M3KgsType = field(
        default=None,
        metadata={
            "name": "CSRP_NDOT",
            "type": "Element",
            "namespace": "",
        },
    )
    csrp_drg: None | M4Kg2Type = field(
        default=None,
        metadata={
            "name": "CSRP_DRG",
            "type": "Element",
            "namespace": "",
        },
    )
    csrp_srp: None | M4Kg2Type = field(
        default=None,
        metadata={
            "name": "CSRP_SRP",
            "type": "Element",
            "namespace": "",
        },
    )
    cthr_r: None | M2S2Type = field(
        default=None,
        metadata={
            "name": "CTHR_R",
            "type": "Element",
            "namespace": "",
        },
    )
    cthr_t: None | M2S2Type = field(
        default=None,
        metadata={
            "name": "CTHR_T",
            "type": "Element",
            "namespace": "",
        },
    )
    cthr_n: None | M2S2Type = field(
        default=None,
        metadata={
            "name": "CTHR_N",
            "type": "Element",
            "namespace": "",
        },
    )
    cthr_rdot: None | M2S3Type = field(
        default=None,
        metadata={
            "name": "CTHR_RDOT",
            "type": "Element",
            "namespace": "",
        },
    )
    cthr_tdot: None | M2S3Type = field(
        default=None,
        metadata={
            "name": "CTHR_TDOT",
            "type": "Element",
            "namespace": "",
        },
    )
    cthr_ndot: None | M2S3Type = field(
        default=None,
        metadata={
            "name": "CTHR_NDOT",
            "type": "Element",
            "namespace": "",
        },
    )
    cthr_drg: None | M3Kgs2Type = field(
        default=None,
        metadata={
            "name": "CTHR_DRG",
            "type": "Element",
            "namespace": "",
        },
    )
    cthr_srp: None | M3Kgs2Type = field(
        default=None,
        metadata={
            "name": "CTHR_SRP",
            "type": "Element",
            "namespace": "",
        },
    )
    cthr_thr: None | M2S4Type = field(
        default=None,
        metadata={
            "name": "CTHR_THR",
            "type": "Element",
            "namespace": "",
        },
    )


@dataclass(kw_only=True)
class CdmStateVectorType:
    class Meta:
        name = "cdmStateVectorType"

    comment: list[str] = field(
        default_factory=list,
        metadata={
            "name": "COMMENT",
            "type": "Element",
            "namespace": "",
        },
    )
    x: CdmPositionType = field(
        metadata={
            "name": "X",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    y: CdmPositionType = field(
        metadata={
            "name": "Y",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    z: CdmPositionType = field(
        metadata={
            "name": "Z",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    x_dot: CdmVelocityType = field(
        metadata={
            "name": "X_DOT",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    y_dot: CdmVelocityType = field(
        metadata={
            "name": "Y_DOT",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    z_dot: CdmVelocityType = field(
        metadata={
            "name": "Z_DOT",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class RelativeStateVectorType:
    class Meta:
        name = "relativeStateVectorType"

    relative_position_r: LengthType = field(
        metadata={
            "name": "RELATIVE_POSITION_R",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    relative_position_t: LengthType = field(
        metadata={
            "name": "RELATIVE_POSITION_T",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    relative_position_n: LengthType = field(
        metadata={
            "name": "RELATIVE_POSITION_N",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    relative_velocity_r: DvType = field(
        metadata={
            "name": "RELATIVE_VELOCITY_R",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    relative_velocity_t: DvType = field(
        metadata={
            "name": "RELATIVE_VELOCITY_T",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    relative_velocity_n: DvType = field(
        metadata={
            "name": "RELATIVE_VELOCITY_N",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class CdmData:
    class Meta:
        name = "cdmData"

    comment: list[str] = field(
        default_factory=list,
        metadata={
            "name": "COMMENT",
            "type": "Element",
            "namespace": "",
        },
    )
    od_parameters: None | OdParametersType = field(
        default=None,
        metadata={
            "name": "odParameters",
            "type": "Element",
            "namespace": "",
        },
    )
    additional_parameters: None | AdditionalParametersType = field(
        default=None,
        metadata={
            "name": "additionalParameters",
            "type": "Element",
            "namespace": "",
        },
    )
    state_vector: CdmStateVectorType = field(
        metadata={
            "name": "stateVector",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    covariance_matrix: CdmCovarianceType = field(
        metadata={
            "name": "covarianceMatrix",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class RelativeMetadataData:
    class Meta:
        name = "relativeMetadataData"

    comment: list[str] = field(
        default_factory=list,
        metadata={
            "name": "COMMENT",
            "type": "Element",
            "namespace": "",
        },
    )
    tca: str = field(
        metadata={
            "name": "TCA",
            "type": "Element",
            "namespace": "",
            "required": True,
            "pattern": r"\-?\d{4}\d*-((\d{2}\-\d{2})|\d{3})T\d{2}:\d{2}:\d{2}(\.\d*)?(Z|[+|\-]\d{2}:\d{2})?|[+|\-]?\d*(\.\d*)?",
        }
    )
    miss_distance: LengthType = field(
        metadata={
            "name": "MISS_DISTANCE",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    relative_speed: None | DvType = field(
        default=None,
        metadata={
            "name": "RELATIVE_SPEED",
            "type": "Element",
            "namespace": "",
        },
    )
    relative_state_vector: None | RelativeStateVectorType = field(
        default=None,
        metadata={
            "name": "relativeStateVector",
            "type": "Element",
            "namespace": "",
        },
    )
    start_screen_period: None | str = field(
        default=None,
        metadata={
            "name": "START_SCREEN_PERIOD",
            "type": "Element",
            "namespace": "",
            "pattern": r"\-?\d{4}\d*-((\d{2}\-\d{2})|\d{3})T\d{2}:\d{2}:\d{2}(\.\d*)?(Z|[+|\-]\d{2}:\d{2})?|[+|\-]?\d*(\.\d*)?",
        },
    )
    stop_screen_period: None | str = field(
        default=None,
        metadata={
            "name": "STOP_SCREEN_PERIOD",
            "type": "Element",
            "namespace": "",
            "pattern": r"\-?\d{4}\d*-((\d{2}\-\d{2})|\d{3})T\d{2}:\d{2}:\d{2}(\.\d*)?(Z|[+|\-]\d{2}:\d{2})?|[+|\-]?\d*(\.\d*)?",
        },
    )
    screen_volume_frame: None | ScreenVolumeFrameType = field(
        default=None,
        metadata={
            "name": "SCREEN_VOLUME_FRAME",
            "type": "Element",
            "namespace": "",
        },
    )
    screen_volume_shape: None | ScreenVolumeShapeType = field(
        default=None,
        metadata={
            "name": "SCREEN_VOLUME_SHAPE",
            "type": "Element",
            "namespace": "",
        },
    )
    screen_volume_x: None | LengthType = field(
        default=None,
        metadata={
            "name": "SCREEN_VOLUME_X",
            "type": "Element",
            "namespace": "",
        },
    )
    screen_volume_y: None | LengthType = field(
        default=None,
        metadata={
            "name": "SCREEN_VOLUME_Y",
            "type": "Element",
            "namespace": "",
        },
    )
    screen_volume_z: None | LengthType = field(
        default=None,
        metadata={
            "name": "SCREEN_VOLUME_Z",
            "type": "Element",
            "namespace": "",
        },
    )
    screen_entry_time: None | str = field(
        default=None,
        metadata={
            "name": "SCREEN_ENTRY_TIME",
            "type": "Element",
            "namespace": "",
            "pattern": r"\-?\d{4}\d*-((\d{2}\-\d{2})|\d{3})T\d{2}:\d{2}:\d{2}(\.\d*)?(Z|[+|\-]\d{2}:\d{2})?|[+|\-]?\d*(\.\d*)?",
        },
    )
    screen_exit_time: None | str = field(
        default=None,
        metadata={
            "name": "SCREEN_EXIT_TIME",
            "type": "Element",
            "namespace": "",
            "pattern": r"\-?\d{4}\d*-((\d{2}\-\d{2})|\d{3})T\d{2}:\d{2}:\d{2}(\.\d*)?(Z|[+|\-]\d{2}:\d{2})?|[+|\-]?\d*(\.\d*)?",
        },
    )
    collision_probability: None | float = field(
        default=None,
        metadata={
            "name": "COLLISION_PROBABILITY",
            "type": "Element",
            "namespace": "",
            "min_inclusive": 0.0,
            "max_inclusive": 1.0,
        },
    )
    collision_probability_method: None | str = field(
        default=None,
        metadata={
            "name": "COLLISION_PROBABILITY_METHOD",
            "type": "Element",
            "namespace": "",
        },
    )


@dataclass(kw_only=True)
class CdmSegment:
    class Meta:
        name = "cdmSegment"

    metadata: CdmMetadata = field(
        metadata={
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    data: CdmData = field(
        metadata={
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class CdmBody:
    class Meta:
        name = "cdmBody"

    relative_metadata_data: RelativeMetadataData = field(
        metadata={
            "name": "relativeMetadataData",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    segment: list[CdmSegment] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "",
            "min_occurs": 2,
            "max_occurs": 2,
        },
    )


@dataclass(kw_only=True)
class CdmType:
    class Meta:
        name = "cdmType"

    header: CdmHeader = field(
        metadata={
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    body: CdmBody = field(
        metadata={
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    id: str = field(
        init=False,
        default="CCSDS_CDM_VERS",
        metadata={
            "type": "Attribute",
            "required": True,
        },
    )
    version: str = field(
        init=False,
        default="1.0",
        metadata={
            "type": "Attribute",
            "required": True,
        },
    )



"""This file was generated by xsdata, v26.1, on 2026-02-18 00:41:07

Generator: DataclassGenerator
See: https://xsdata.readthedocs.io/
"""

from __future__ import annotations

from dataclasses import dataclass, field
from enum import Enum

__NAMESPACE__ = "urn:ccsds:schema:ndmxml"


class AccUnits(Enum):
    KM_S_2 = "km/s**2"


class AcmAttitudeType(Enum):
    QUATERNION = "QUATERNION"
    QUATERNION_1 = "quaternion"
    EULER_ANGLES = "EULER_ANGLES"
    EULER_ANGLES_1 = "euler_angles"
    DCM = "DCM"
    DCM_1 = "dcm"
    ANGVEL = "ANGVEL"
    ANGVEL_1 = "angvel"
    Q_DOT = "Q_DOT"
    Q_DOT_1 = "q_dot"
    EULER_RATE = "EULER_RATE"
    EULER_RATE_1 = "euler_rate"
    GYRO_BIAS = "GYRO_BIAS"
    GYRO_BIAS_1 = "gyro_bias"


class AcmCovarianceLineType(Enum):
    ANGLE = "ANGLE"
    ANGLE_1 = "angle"
    ANGLE_GYROBIAS = "ANGLE_GYROBIAS"
    ANGLE_GYROBIAS_1 = "angle_gyrobias"
    ANGLE_ANGVEL = "ANGLE_ANGVEL"
    ANGLE_ANGVEL_1 = "angle_angvel"
    QUATERNION = "QUATERNION"
    QUATERNION_1 = "quaternion"
    QUATERNION_GYROBIAS = "QUATERNION_GYROBIAS"
    QUATERNION_GYROBIAS_1 = "quaternion_gyrobias"
    QUATERNION_ANGVEL = "QUATERNION_ANGVEL"
    QUATERNION_ANGVEL_1 = "quaternion_angvel"


class AdMethodType(Enum):
    EKF = "EKF"
    EKF_1 = "ekf"
    TRIAD = "TRIAD"
    TRIAD_1 = "triad"
    QUEST = "QUEST"
    QUEST_1 = "quest"
    BATCH = "BATCH"
    BATCH_1 = "batch"
    Q_METHOD = "Q_METHOD"
    Q_METHOD_1 = "q_method"
    FILTER_SMOOTHER = "FILTER_SMOOTHER"
    FILTER_SMOOTHER_1 = "filter_smoother"
    OTHER = "OTHER"
    OTHER_1 = "other"


@dataclass(kw_only=True)
class AdmHeader:
    class Meta:
        name = "admHeader"

    comment: list[str] = field(
        default_factory=list,
        metadata={
            "name": "COMMENT",
            "type": "Element",
            "namespace": "",
        },
    )
    classification: None | str = field(
        default=None,
        metadata={
            "name": "CLASSIFICATION",
            "type": "Element",
            "namespace": "",
        },
    )
    creation_date: str = field(
        metadata={
            "name": "CREATION_DATE",
            "type": "Element",
            "namespace": "",
            "required": True,
            "pattern": r"\-?\d{4}\d*-((\d{2}\-\d{2})|\d{3})T\d{2}:\d{2}:\d{2}(\.\d*)?(Z|[+|\-]\d{2}:\d{2})?|[+|\-]?\d*(\.\d*)?",
        }
    )
    originator: str = field(
        metadata={
            "name": "ORIGINATOR",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    message_id: None | str = field(
        default=None,
        metadata={
            "name": "MESSAGE_ID",
            "type": "Element",
            "namespace": "",
        },
    )


class AngMomentumUnits(Enum):
    N_M_S = "N*m*s"


class AngleRateUnits(Enum):
    DEG_S = "deg/s"


class AngleUnits(Enum):
    DEG = "deg"


class AreaUnits(Enum):
    M_2 = "m**2"


class AttBasisType(Enum):
    PREDICTED = "PREDICTED"
    PREDICTED_1 = "predicted"
    DETERMINED_GND = "DETERMINED_GND"
    DETERMINED_GND_1 = "determined_gnd"
    DETERMINED_OBC = "DETERMINED_OBC"
    DETERMINED_OBC_1 = "determined_obc"
    SIMULATED = "SIMULATED"
    SIMULATED_1 = "simulated"


class AttRateType(Enum):
    ANGVEL = "ANGVEL"
    ANGVEL_1 = "angvel"
    Q_DOT = "Q_DOT"
    Q_DOT_1 = "q_dot"
    EULER_RATE = "EULER_RATE"
    EULER_RATE_1 = "euler_rate"
    GYRO_BIAS = "GYRO_BIAS"
    GYRO_BIAS_1 = "gyro_bias"


class AttitudeTypeType(Enum):
    QUATERNION = "quaternion"
    QUATERNION_1 = "QUATERNION"
    QUATERNION_DERIVATIVE = "quaternion/derivative"
    QUATERNION_DERIVATIVE_1 = "QUATERNION/DERIVATIVE"
    QUATERNION_ANGVEL = "quaternion/angvel"
    QUATERNION_ANGVEL_1 = "QUATERNION/ANGVEL"
    EULER_ANGLE = "euler_angle"
    EULER_ANGLE_1 = "EULER_ANGLE"
    EULER_ANGLE_DERIVATIVE = "euler_angle/derivative"
    EULER_ANGLE_DERIVATIVE_1 = "EULER_ANGLE/DERIVATIVE"
    EULER_ANGLE_ANGVEL = "euler_angle/angvel"
    EULER_ANGLE_ANGVEL_1 = "EULER_ANGLE/ANGVEL"
    SPIN = "spin"
    SPIN_1 = "SPIN"
    SPIN_NUTATION = "spin/nutation"
    SPIN_NUTATION_1 = "SPIN/NUTATION"
    SPIN_NUTATION_MOM = "spin/nutation_mom"
    SPIN_NUTATION_MOM_1 = "SPIN/NUTATION_MOM"


class BallisticCoeffUnitsType(Enum):
    KG_M_2 = "kg/m**2"


class ControlledType(Enum):
    YES = "YES"
    YES_1 = "yes"
    NO = "NO"
    NO_1 = "no"
    UNKNOWN = "UNKNOWN"
    UNKNOWN_1 = "unknown"


class CovBasisType(Enum):
    PREDICTED = "PREDICTED"
    DETERMINED = "DETERMINED"
    EMPIRICAL = "EMPIRICAL"
    SIMULATED = "SIMULATED"
    OTHER = "OTHER"


class CovOrderType(Enum):
    LTM = "LTM"
    UTM = "UTM"
    FULL = "FULL"
    LTMWCC = "LTMWCC"
    UTMWCC = "UTMWCC"


class DayIntervalUnits(Enum):
    D = "d"


class DisintegrationType(Enum):
    NONE = "NONE"
    MASS_LOSS = "MASS-LOSS"
    BREAK_UP = "BREAK-UP"
    MASS_LOSS_BREAK_UP = "MASS-LOSS + BREAK-UP"


class FrequencyUnits(Enum):
    HZ = "Hz"


class GeomagUnits(Enum):
    N_T = "nT"


class GmUnits(Enum):
    KM_3_S_2 = "km**3/s**2"
    KM_3_S_2_1 = "KM**3/S**2"


class ImpactUncertaintyType(Enum):
    NONE = "NONE"
    ANALYTICAL = "ANALYTICAL"
    STOCHASTIC = "STOCHASTIC"
    EMPIRICAL = "EMPIRICAL"


class Km2Units(Enum):
    KM_2 = "km**2"


class Km2S2Units(Enum):
    KM_2_S_2 = "km**2/s**2"


class Km2SUnits(Enum):
    KM_2_S = "km**2/s"


class LatLonUnits(Enum):
    DEG = "deg"


class LengthUnits(Enum):
    M = "m"


class ManBasisType(Enum):
    CANDIDATE = "CANDIDATE"
    PLANNED = "PLANNED"
    ANTICIPATED = "ANTICIPATED"
    TELEMETRY = "TELEMETRY"
    DETERMINED = "DETERMINED"
    SIMULATED = "SIMULATED"
    OTHER = "OTHER"


class ManDctype(Enum):
    CONTINUOUS = "CONTINUOUS"
    TIME = "TIME"
    TIME_AND_ANGLE = "TIME_AND_ANGLE"


class MassUnits(Enum):
    KG = "kg"


class MomentUnits(Enum):
    KG_M_2 = "kg*m**2"


class Ms2Units(Enum):
    M_S_2 = "m/s**2"


@dataclass(kw_only=True)
class NdmHeader:
    class Meta:
        name = "ndmHeader"

    comment: list[str] = field(
        default_factory=list,
        metadata={
            "name": "COMMENT",
            "type": "Element",
            "namespace": "",
        },
    )
    creation_date: str = field(
        metadata={
            "name": "CREATION_DATE",
            "type": "Element",
            "namespace": "",
            "required": True,
            "pattern": r"\-?\d{4}\d*-((\d{2}\-\d{2})|\d{3})T\d{2}:\d{2}:\d{2}(\.\d*)?(Z|[+|\-]\d{2}:\d{2})?|[+|\-]?\d*(\.\d*)?",
        }
    )
    originator: str = field(
        metadata={
            "name": "ORIGINATOR",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )


class NumPerYearUnits(Enum):
    YR = "#/yr"


class ObjectDescriptionType(Enum):
    PAYLOAD = "PAYLOAD"
    PAYLOAD_1 = "payload"
    ROCKET_BODY = "ROCKET BODY"
    ROCKET_BODY_1 = "rocket body"
    DEBRIS = "DEBRIS"
    DEBRIS_1 = "debris"
    UNKNOWN = "UNKNOWN"
    UNKNOWN_1 = "unknown"
    OTHER = "OTHER"
    OTHER_1 = "other"


@dataclass(kw_only=True)
class OdmHeader:
    class Meta:
        name = "odmHeader"

    comment: list[str] = field(
        default_factory=list,
        metadata={
            "name": "COMMENT",
            "type": "Element",
            "namespace": "",
        },
    )
    classification: None | str = field(
        default=None,
        metadata={
            "name": "CLASSIFICATION",
            "type": "Element",
            "namespace": "",
        },
    )
    creation_date: str = field(
        metadata={
            "name": "CREATION_DATE",
            "type": "Element",
            "namespace": "",
            "required": True,
            "pattern": r"\-?\d{4}\d*-((\d{2}\-\d{2})|\d{3})T\d{2}:\d{2}:\d{2}(\.\d*)?(Z|[+|\-]\d{2}:\d{2})?|[+|\-]?\d*(\.\d*)?",
        }
    )
    originator: str = field(
        metadata={
            "name": "ORIGINATOR",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    message_id: None | str = field(
        default=None,
        metadata={
            "name": "MESSAGE_ID",
            "type": "Element",
            "namespace": "",
        },
    )


@dataclass(kw_only=True)
class OemCovarianceMatrixAbstractType:
    class Meta:
        name = "oemCovarianceMatrixAbstractType"

    comment: list[str] = field(
        default_factory=list,
        metadata={
            "name": "COMMENT",
            "type": "Element",
            "namespace": "",
        },
    )
    epoch: str = field(
        metadata={
            "name": "EPOCH",
            "type": "Element",
            "namespace": "",
            "required": True,
            "pattern": r"\-?\d{4}\d*-((\d{2}\-\d{2})|\d{3})T\d{2}:\d{2}:\d{2}(\.\d*)?(Z|[+|\-]\d{2}:\d{2})?|[+|\-]?\d*(\.\d*)?",
        }
    )
    cov_ref_frame: None | str = field(
        default=None,
        metadata={
            "name": "COV_REF_FRAME",
            "type": "Element",
            "namespace": "",
        },
    )


@dataclass(kw_only=True)
class OpmCovarianceMatrixAbstractType:
    class Meta:
        name = "opmCovarianceMatrixAbstractType"

    comment: list[str] = field(
        default_factory=list,
        metadata={
            "name": "COMMENT",
            "type": "Element",
            "namespace": "",
        },
    )
    cov_ref_frame: None | str = field(
        default=None,
        metadata={
            "name": "COV_REF_FRAME",
            "type": "Element",
            "namespace": "",
        },
    )


class PercentageUnits(Enum):
    PERCENT_SIGN = "%"


class PositionCovarianceUnits(Enum):
    KM_2 = "km**2"


class PositionUnits(Enum):
    KM = "km"


class PositionVelocityCovarianceUnits(Enum):
    KM_2_S = "km**2/s"


class QuaternionDotUnits(Enum):
    VALUE_1_S = "1/s"


@dataclass(kw_only=True)
class QuaternionType:
    class Meta:
        name = "quaternionType"

    q1: float = field(
        metadata={
            "name": "Q1",
            "type": "Element",
            "namespace": "",
            "required": True,
            "min_inclusive": -1.0,
            "max_inclusive": 1.0,
        }
    )
    q2: float = field(
        metadata={
            "name": "Q2",
            "type": "Element",
            "namespace": "",
            "required": True,
            "min_inclusive": -1.0,
            "max_inclusive": 1.0,
        }
    )
    q3: float = field(
        metadata={
            "name": "Q3",
            "type": "Element",
            "namespace": "",
            "required": True,
            "min_inclusive": -1.0,
            "max_inclusive": 1.0,
        }
    )
    qc: float = field(
        metadata={
            "name": "QC",
            "type": "Element",
            "namespace": "",
            "required": True,
            "min_inclusive": -1.0,
            "max_inclusive": 1.0,
        }
    )


class ReentryUncertaintyMethodType(Enum):
    NONE = "NONE"
    ANALYTICAL = "ANALYTICAL"
    STOCHASTIC = "STOCHASTIC"
    EMPIRICAL = "EMPIRICAL"


class RevNumBasisType(Enum):
    VALUE_0 = 0
    VALUE_1 = 1


class RotseqType(Enum):
    XYX = "XYX"
    XYZ = "XYZ"
    XZX = "XZX"
    XZY = "XZY"
    YXY = "YXY"
    YXZ = "YXZ"
    YZX = "YZX"
    YZY = "YZY"
    ZXY = "ZXY"
    ZXZ = "ZXZ"
    ZYX = "ZYX"
    ZYZ = "ZYZ"


class SigmaUunits(Enum):
    DEG_S_1_5 = "deg/s**1.5"


class SigmaVunits(Enum):
    DEG_S_0_5 = "deg/s**0.5"


class SolarFluxUnits(Enum):
    SFU = "SFU"
    VALUE_10_4_JANSKY = "10**4 Jansky"
    VALUE_10_22_W_M_2_HZ = "10**-22 W/(m**2/Hz)"
    VALUE_10_19_ERG_S_CM_2_HZ = "10**-19 erg/(s*cm**2*Hz)"


class ThrustUnits(Enum):
    N = "N"


class TimeUnits(Enum):
    S = "s"


class TorqueUnits(Enum):
    N_M = "N*m"


class TrajBasisType(Enum):
    PREDICTED = "PREDICTED"
    DETERMINED = "DETERMINED"
    TELEMETRY = "TELEMETRY"
    SIMULATED = "SIMULATED"
    OTHER = "OTHER"


@dataclass(kw_only=True)
class UserDefinedParameterType:
    class Meta:
        name = "userDefinedParameterType"

    value: str = field(
        default="",
        metadata={
            "required": True,
        },
    )
    parameter: str = field(
        metadata={
            "type": "Attribute",
            "required": True,
        }
    )


class VelocityCovarianceUnits(Enum):
    KM_2_S_2 = "km**2/s**2"


class VelocityUnits(Enum):
    KM_S = "km/s"


class WkgUnits(Enum):
    W_KG = "W/kg"


class YesNoType(Enum):
    YES = "YES"
    YES_1 = "yes"
    NO = "NO"
    NO_1 = "no"


@dataclass(kw_only=True)
class AccType:
    class Meta:
        name = "accType"

    value: float = field(
        metadata={
            "required": True,
        }
    )
    units: None | AccUnits = field(
        default=None,
        metadata={
            "type": "Attribute",
        },
    )


@dataclass(kw_only=True)
class AltType:
    class Meta:
        name = "altType"

    value: float = field(
        metadata={
            "required": True,
            "min_inclusive": -430.5,
            "max_inclusive": 8848.0,
        }
    )
    units: None | LengthUnits = field(
        default=None,
        metadata={
            "type": "Attribute",
        },
    )


@dataclass(kw_only=True)
class AngMomentumType:
    class Meta:
        name = "angMomentumType"

    value: float = field(
        metadata={
            "required": True,
        }
    )
    units: None | AngMomentumUnits = field(
        default=None,
        metadata={
            "type": "Attribute",
        },
    )


@dataclass(kw_only=True)
class AngVelComponentType:
    class Meta:
        name = "angVelComponentType"

    value: float = field(
        metadata={
            "required": True,
        }
    )
    units: None | AngleRateUnits = field(
        default=None,
        metadata={
            "type": "Attribute",
        },
    )


@dataclass(kw_only=True)
class AngleRateType:
    class Meta:
        name = "angleRateType"

    value: float = field(
        metadata={
            "required": True,
        }
    )
    units: None | AngleRateUnits = field(
        default=None,
        metadata={
            "type": "Attribute",
        },
    )


@dataclass(kw_only=True)
class AngleType:
    class Meta:
        name = "angleType"

    value: float = field(
        metadata={
            "required": True,
            "min_inclusive": -360.0,
            "max_exclusive": 360.0,
        }
    )
    units: None | AngleUnits = field(
        default=None,
        metadata={
            "type": "Attribute",
        },
    )


@dataclass(kw_only=True)
class AreaType:
    class Meta:
        name = "areaType"

    value: float = field(
        metadata={
            "required": True,
            "min_inclusive": 0.0,
        }
    )
    units: None | AreaUnits = field(
        default=None,
        metadata={
            "type": "Attribute",
        },
    )


@dataclass(kw_only=True)
class BallisticCoeffType:
    class Meta:
        name = "ballisticCoeffType"

    value: float = field(
        metadata={
            "required": True,
            "min_inclusive": 0.0,
        }
    )
    units: None | BallisticCoeffUnitsType = field(
        default=None,
        metadata={
            "type": "Attribute",
        },
    )


@dataclass(kw_only=True)
class CpType:
    class Meta:
        name = "cpType"

    value: list[float] = field(
        default_factory=list,
        metadata={
            "length": 3,
            "tokens": True,
        },
    )
    units: None | LengthUnits = field(
        default=None,
        metadata={
            "type": "Attribute",
        },
    )


@dataclass(kw_only=True)
class DayIntervalTypeUo:
    class Meta:
        name = "dayIntervalTypeUO"

    value: float = field(
        metadata={
            "required": True,
            "min_inclusive": 0.0,
        }
    )
    units: None | DayIntervalUnits = field(
        default=None,
        metadata={
            "type": "Attribute",
        },
    )


@dataclass(kw_only=True)
class DayIntervalTypeUr:
    class Meta:
        name = "dayIntervalTypeUR"

    value: float = field(
        metadata={
            "required": True,
            "min_exclusive": 0.0,
        }
    )
    units: DayIntervalUnits = field(
        metadata={
            "type": "Attribute",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class DeltamassType:
    class Meta:
        name = "deltamassType"

    value: float = field(
        metadata={
            "required": True,
            "max_exclusive": 0.0,
        }
    )
    units: None | MassUnits = field(
        default=None,
        metadata={
            "type": "Attribute",
        },
    )


@dataclass(kw_only=True)
class DeltamassTypeZ:
    class Meta:
        name = "deltamassTypeZ"

    value: float = field(
        metadata={
            "required": True,
            "max_inclusive": 0.0,
        }
    )
    units: None | MassUnits = field(
        default=None,
        metadata={
            "type": "Attribute",
        },
    )


@dataclass(kw_only=True)
class DistanceType:
    class Meta:
        name = "distanceType"

    value: float = field(
        metadata={
            "required": True,
        }
    )
    units: None | PositionUnits = field(
        default=None,
        metadata={
            "type": "Attribute",
        },
    )


@dataclass(kw_only=True)
class DurationType:
    class Meta:
        name = "durationType"

    value: float = field(
        metadata={
            "required": True,
            "min_inclusive": 0.0,
        }
    )
    units: None | TimeUnits = field(
        default=None,
        metadata={
            "type": "Attribute",
        },
    )


@dataclass(kw_only=True)
class FrequencyType:
    class Meta:
        name = "frequencyType"

    value: float = field(
        metadata={
            "required": True,
            "min_exclusive": 0.0,
        }
    )
    units: None | FrequencyUnits = field(
        default=None,
        metadata={
            "type": "Attribute",
        },
    )


@dataclass(kw_only=True)
class GeomagType:
    class Meta:
        name = "geomagType"

    value: float = field(
        metadata={
            "required": True,
        }
    )
    units: None | GeomagUnits = field(
        default=None,
        metadata={
            "type": "Attribute",
        },
    )


@dataclass(kw_only=True)
class GmType:
    class Meta:
        name = "gmType"

    value: float = field(
        metadata={
            "required": True,
            "min_exclusive": 0.0,
        }
    )
    units: None | GmUnits = field(
        default=None,
        metadata={
            "type": "Attribute",
        },
    )


@dataclass(kw_only=True)
class InclinationType:
    class Meta:
        name = "inclinationType"

    value: float = field(
        metadata={
            "required": True,
            "min_inclusive": 0.0,
            "max_exclusive": 360.0,
            "max_inclusive": 180.0,
        }
    )
    units: None | AngleUnits = field(
        default=None,
        metadata={
            "type": "Attribute",
        },
    )


@dataclass(kw_only=True)
class Km2Type:
    class Meta:
        name = "km2Type"

    value: float = field(
        metadata={
            "required": True,
        }
    )
    units: None | Km2Units = field(
        default=None,
        metadata={
            "type": "Attribute",
        },
    )


@dataclass(kw_only=True)
class Km2S2Type:
    class Meta:
        name = "km2s2Type"

    value: float = field(
        metadata={
            "required": True,
        }
    )
    units: None | Km2S2Units = field(
        default=None,
        metadata={
            "type": "Attribute",
        },
    )


@dataclass(kw_only=True)
class Km2SType:
    class Meta:
        name = "km2sType"

    value: float = field(
        metadata={
            "required": True,
        }
    )
    units: None | Km2SUnits = field(
        default=None,
        metadata={
            "type": "Attribute",
        },
    )


@dataclass(kw_only=True)
class LatType:
    class Meta:
        name = "latType"

    value: float = field(
        metadata={
            "required": True,
            "min_inclusive": -90.0,
            "max_inclusive": 90.0,
        }
    )
    units: LatLonUnits = field(
        metadata={
            "type": "Attribute",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class LengthTypeUo:
    class Meta:
        name = "lengthTypeUO"

    value: float = field(
        metadata={
            "required": True,
        }
    )
    units: None | LengthUnits = field(
        default=None,
        metadata={
            "type": "Attribute",
        },
    )


@dataclass(kw_only=True)
class LengthTypeUr:
    class Meta:
        name = "lengthTypeUR"

    value: float = field(
        metadata={
            "required": True,
        }
    )
    units: LengthUnits = field(
        metadata={
            "type": "Attribute",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class LonType:
    class Meta:
        name = "lonType"

    value: float = field(
        metadata={
            "required": True,
            "min_inclusive": -180.0,
            "max_inclusive": 180.0,
        }
    )
    units: LatLonUnits = field(
        metadata={
            "type": "Attribute",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class ManeuverFreqType:
    class Meta:
        name = "maneuverFreqType"

    value: float = field(
        metadata={
            "required": True,
        }
    )
    units: None | NumPerYearUnits = field(
        default=None,
        metadata={
            "type": "Attribute",
        },
    )


@dataclass(kw_only=True)
class MassType:
    class Meta:
        name = "massType"

    value: float = field(
        metadata={
            "required": True,
            "min_inclusive": 0.0,
        }
    )
    units: None | MassUnits = field(
        default=None,
        metadata={
            "type": "Attribute",
        },
    )


@dataclass(kw_only=True)
class MomentType:
    class Meta:
        name = "momentType"

    value: float = field(
        metadata={
            "required": True,
        }
    )
    units: None | MomentUnits = field(
        default=None,
        metadata={
            "type": "Attribute",
        },
    )


@dataclass(kw_only=True)
class Ms2Type:
    class Meta:
        name = "ms2Type"

    value: float = field(
        metadata={
            "required": True,
        }
    )
    units: Ms2Units = field(
        metadata={
            "type": "Attribute",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class PercentageTypeUo:
    class Meta:
        name = "percentageTypeUO"

    value: float = field(
        metadata={
            "required": True,
            "min_inclusive": 0.0,
            "max_inclusive": 100.0,
        }
    )
    units: None | PercentageUnits = field(
        default=None,
        metadata={
            "type": "Attribute",
        },
    )


@dataclass(kw_only=True)
class PercentageTypeUr:
    class Meta:
        name = "percentageTypeUR"

    value: float = field(
        metadata={
            "required": True,
            "min_inclusive": 0.0,
            "max_inclusive": 100.0,
        }
    )
    units: PercentageUnits = field(
        metadata={
            "type": "Attribute",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class PositionCovarianceType:
    class Meta:
        name = "positionCovarianceType"

    value: float = field(
        metadata={
            "required": True,
        }
    )
    units: None | PositionCovarianceUnits = field(
        default=None,
        metadata={
            "type": "Attribute",
        },
    )


@dataclass(kw_only=True)
class PositionTypeUo:
    class Meta:
        name = "positionTypeUO"

    value: float = field(
        metadata={
            "required": True,
        }
    )
    units: None | PositionUnits = field(
        default=None,
        metadata={
            "type": "Attribute",
        },
    )


@dataclass(kw_only=True)
class PositionTypeUr:
    class Meta:
        name = "positionTypeUR"

    value: float = field(
        metadata={
            "required": True,
        }
    )
    units: PositionUnits = field(
        metadata={
            "type": "Attribute",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class PositionVelocityCovarianceType:
    class Meta:
        name = "positionVelocityCovarianceType"

    value: float = field(
        metadata={
            "required": True,
        }
    )
    units: None | PositionVelocityCovarianceUnits = field(
        default=None,
        metadata={
            "type": "Attribute",
        },
    )


@dataclass(kw_only=True)
class QuaternionDotComponentType:
    class Meta:
        name = "quaternionDotComponentType"

    value: float = field(
        metadata={
            "required": True,
        }
    )
    units: None | QuaternionDotUnits = field(
        default=None,
        metadata={
            "type": "Attribute",
        },
    )


@dataclass(kw_only=True)
class QuaternionEphemerisType:
    class Meta:
        name = "quaternionEphemerisType"

    epoch: str = field(
        metadata={
            "name": "EPOCH",
            "type": "Element",
            "namespace": "",
            "required": True,
            "pattern": r"\-?\d{4}\d*-((\d{2}\-\d{2})|\d{3})T\d{2}:\d{2}:\d{2}(\.\d*)?(Z|[+|\-]\d{2}:\d{2})?|[+|\-]?\d*(\.\d*)?",
        }
    )
    quaternion: QuaternionType = field(
        metadata={
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class RelTimeType:
    class Meta:
        name = "relTimeType"

    value: float = field(
        metadata={
            "required": True,
        }
    )
    units: None | TimeUnits = field(
        default=None,
        metadata={
            "type": "Attribute",
        },
    )


@dataclass(kw_only=True)
class SensorNoiseType:
    class Meta:
        name = "sensorNoiseType"

    value: str = field(
        default="",
        metadata={
            "required": True,
        },
    )
    units: None | AngleUnits = field(
        default=None,
        metadata={
            "type": "Attribute",
        },
    )


@dataclass(kw_only=True)
class SigmaUtype:
    class Meta:
        name = "sigmaUType"

    value: float = field(
        metadata={
            "required": True,
        }
    )
    units: None | SigmaUunits = field(
        default=None,
        metadata={
            "type": "Attribute",
        },
    )


@dataclass(kw_only=True)
class SigmaVtype:
    class Meta:
        name = "sigmaVType"

    value: float = field(
        metadata={
            "required": True,
        }
    )
    units: None | SigmaVunits = field(
        default=None,
        metadata={
            "type": "Attribute",
        },
    )


@dataclass(kw_only=True)
class SolarFluxType:
    class Meta:
        name = "solarFluxType"

    value: float = field(
        metadata={
            "required": True,
        }
    )
    units: None | SolarFluxUnits = field(
        default=None,
        metadata={
            "type": "Attribute",
        },
    )


@dataclass(kw_only=True)
class TargetMomentumType:
    class Meta:
        name = "targetMomentumType"

    value: list[float] = field(
        default_factory=list,
        metadata={
            "length": 3,
            "tokens": True,
        },
    )
    units: None | AngMomentumUnits = field(
        default=None,
        metadata={
            "type": "Attribute",
        },
    )


@dataclass(kw_only=True)
class ThrustType:
    class Meta:
        name = "thrustType"

    value: float = field(
        metadata={
            "required": True,
        }
    )
    units: None | ThrustUnits = field(
        default=None,
        metadata={
            "type": "Attribute",
        },
    )


@dataclass(kw_only=True)
class TimeOffsetType:
    class Meta:
        name = "timeOffsetType"

    value: float = field(
        metadata={
            "required": True,
        }
    )
    units: None | TimeUnits = field(
        default=None,
        metadata={
            "type": "Attribute",
        },
    )


@dataclass(kw_only=True)
class TorqueType:
    class Meta:
        name = "torqueType"

    value: float = field(
        metadata={
            "required": True,
        }
    )
    units: None | TorqueUnits = field(
        default=None,
        metadata={
            "type": "Attribute",
        },
    )


@dataclass(kw_only=True)
class UserDefinedType:
    class Meta:
        name = "userDefinedType"

    comment: list[str] = field(
        default_factory=list,
        metadata={
            "name": "COMMENT",
            "type": "Element",
            "namespace": "",
            "sequence": 1,
        },
    )
    user_defined: list[UserDefinedParameterType] = field(
        default_factory=list,
        metadata={
            "name": "USER_DEFINED",
            "type": "Element",
            "namespace": "",
            "sequence": 1,
        },
    )


@dataclass(kw_only=True)
class VelocityCovarianceType:
    class Meta:
        name = "velocityCovarianceType"

    value: float = field(
        metadata={
            "required": True,
        }
    )
    units: None | VelocityCovarianceUnits = field(
        default=None,
        metadata={
            "type": "Attribute",
        },
    )


@dataclass(kw_only=True)
class VelocityTypeUo:
    class Meta:
        name = "velocityTypeUO"

    value: float = field(
        metadata={
            "required": True,
        }
    )
    units: None | VelocityUnits = field(
        default=None,
        metadata={
            "type": "Attribute",
        },
    )


@dataclass(kw_only=True)
class VelocityTypeUr:
    class Meta:
        name = "velocityTypeUR"

    value: float = field(
        metadata={
            "required": True,
        }
    )
    units: VelocityUnits = field(
        metadata={
            "type": "Attribute",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class WkgType:
    class Meta:
        name = "wkgType"

    value: float = field(
        metadata={
            "required": True,
            "min_inclusive": 0.0,
        }
    )
    units: WkgUnits = field(
        metadata={
            "type": "Attribute",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class AngVelStateType:
    class Meta:
        name = "angVelStateType"

    comment: list[str] = field(
        default_factory=list,
        metadata={
            "name": "COMMENT",
            "type": "Element",
            "namespace": "",
        },
    )
    ref_frame_a: str = field(
        metadata={
            "name": "REF_FRAME_A",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    ref_frame_b: str = field(
        metadata={
            "name": "REF_FRAME_B",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    angvel_frame: str = field(
        metadata={
            "name": "ANGVEL_FRAME",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    angvel_x: AngVelComponentType = field(
        metadata={
            "name": "ANGVEL_X",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    angvel_y: AngVelComponentType = field(
        metadata={
            "name": "ANGVEL_Y",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    angvel_z: AngVelComponentType = field(
        metadata={
            "name": "ANGVEL_Z",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class AngVelType:
    class Meta:
        name = "angVelType"

    angvel_x: AngVelComponentType = field(
        metadata={
            "name": "ANGVEL_X",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    angvel_y: AngVelComponentType = field(
        metadata={
            "name": "ANGVEL_Y",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    angvel_z: AngVelComponentType = field(
        metadata={
            "name": "ANGVEL_Z",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class AtmosphericReentryParametersType:
    class Meta:
        name = "atmosphericReentryParametersType"

    comment: list[str] = field(
        default_factory=list,
        metadata={
            "name": "COMMENT",
            "type": "Element",
            "namespace": "",
        },
    )
    orbit_lifetime: DayIntervalTypeUr = field(
        metadata={
            "name": "ORBIT_LIFETIME",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    reentry_altitude: PositionTypeUr = field(
        metadata={
            "name": "REENTRY_ALTITUDE",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    orbit_lifetime_window_start: None | DayIntervalTypeUr = field(
        default=None,
        metadata={
            "name": "ORBIT_LIFETIME_WINDOW_START",
            "type": "Element",
            "namespace": "",
        },
    )
    orbit_lifetime_window_end: None | DayIntervalTypeUr = field(
        default=None,
        metadata={
            "name": "ORBIT_LIFETIME_WINDOW_END",
            "type": "Element",
            "namespace": "",
        },
    )
    nominal_reentry_epoch: None | str = field(
        default=None,
        metadata={
            "name": "NOMINAL_REENTRY_EPOCH",
            "type": "Element",
            "namespace": "",
            "pattern": r"\-?\d{4}\d*-((\d{2}\-\d{2})|\d{3})T\d{2}:\d{2}:\d{2}(\.\d*)?(Z|[+|\-]\d{2}:\d{2})?|[+|\-]?\d*(\.\d*)?",
        },
    )
    reentry_window_start: None | str = field(
        default=None,
        metadata={
            "name": "REENTRY_WINDOW_START",
            "type": "Element",
            "namespace": "",
            "pattern": r"\-?\d{4}\d*-((\d{2}\-\d{2})|\d{3})T\d{2}:\d{2}:\d{2}(\.\d*)?(Z|[+|\-]\d{2}:\d{2})?|[+|\-]?\d*(\.\d*)?",
        },
    )
    reentry_window_end: None | str = field(
        default=None,
        metadata={
            "name": "REENTRY_WINDOW_END",
            "type": "Element",
            "namespace": "",
            "pattern": r"\-?\d{4}\d*-((\d{2}\-\d{2})|\d{3})T\d{2}:\d{2}:\d{2}(\.\d*)?(Z|[+|\-]\d{2}:\d{2})?|[+|\-]?\d*(\.\d*)?",
        },
    )
    orbit_lifetime_confidence_level: None | PercentageTypeUr = field(
        default=None,
        metadata={
            "name": "ORBIT_LIFETIME_CONFIDENCE_LEVEL",
            "type": "Element",
            "namespace": "",
        },
    )


@dataclass(kw_only=True)
class AttManeuverStateType:
    class Meta:
        name = "attManeuverStateType"

    comment: list[str] = field(
        default_factory=list,
        metadata={
            "name": "COMMENT",
            "type": "Element",
            "namespace": "",
        },
    )
    man_epoch_start: str = field(
        metadata={
            "name": "MAN_EPOCH_START",
            "type": "Element",
            "namespace": "",
            "required": True,
            "pattern": r"\-?\d{4}\d*-((\d{2}\-\d{2})|\d{3})T\d{2}:\d{2}:\d{2}(\.\d*)?(Z|[+|\-]\d{2}:\d{2})?|[+|\-]?\d*(\.\d*)?",
        }
    )
    man_duration: DurationType = field(
        metadata={
            "name": "MAN_DURATION",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    man_ref_frame: str = field(
        metadata={
            "name": "MAN_REF_FRAME",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    man_tor_x: TorqueType = field(
        metadata={
            "name": "MAN_TOR_X",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    man_tor_y: TorqueType = field(
        metadata={
            "name": "MAN_TOR_Y",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    man_tor_z: TorqueType = field(
        metadata={
            "name": "MAN_TOR_Z",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    man_delta_mass: None | DeltamassTypeZ = field(
        default=None,
        metadata={
            "name": "MAN_DELTA_MASS",
            "type": "Element",
            "namespace": "",
        },
    )


@dataclass(kw_only=True)
class EulerAngleAngVelType:
    class Meta:
        name = "eulerAngleAngVelType"

    epoch: str = field(
        metadata={
            "name": "EPOCH",
            "type": "Element",
            "namespace": "",
            "required": True,
            "pattern": r"\-?\d{4}\d*-((\d{2}\-\d{2})|\d{3})T\d{2}:\d{2}:\d{2}(\.\d*)?(Z|[+|\-]\d{2}:\d{2})?|[+|\-]?\d*(\.\d*)?",
        }
    )
    angle_1: AngleType = field(
        metadata={
            "name": "ANGLE_1",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    angle_2: AngleType = field(
        metadata={
            "name": "ANGLE_2",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    angle_3: AngleType = field(
        metadata={
            "name": "ANGLE_3",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    angvel_x: AngVelComponentType = field(
        metadata={
            "name": "ANGVEL_X",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    angvel_y: AngVelComponentType = field(
        metadata={
            "name": "ANGVEL_Y",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    angvel_z: AngVelComponentType = field(
        metadata={
            "name": "ANGVEL_Z",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class EulerAngleDerivativeType:
    class Meta:
        name = "eulerAngleDerivativeType"

    epoch: str = field(
        metadata={
            "name": "EPOCH",
            "type": "Element",
            "namespace": "",
            "required": True,
            "pattern": r"\-?\d{4}\d*-((\d{2}\-\d{2})|\d{3})T\d{2}:\d{2}:\d{2}(\.\d*)?(Z|[+|\-]\d{2}:\d{2})?|[+|\-]?\d*(\.\d*)?",
        }
    )
    angle_1: AngleType = field(
        metadata={
            "name": "ANGLE_1",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    angle_2: AngleType = field(
        metadata={
            "name": "ANGLE_2",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    angle_3: AngleType = field(
        metadata={
            "name": "ANGLE_3",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    angle_1_dot: AngleRateType = field(
        metadata={
            "name": "ANGLE_1_DOT",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    angle_2_dot: AngleRateType = field(
        metadata={
            "name": "ANGLE_2_DOT",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    angle_3_dot: AngleRateType = field(
        metadata={
            "name": "ANGLE_3_DOT",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class EulerAngleStateType:
    class Meta:
        name = "eulerAngleStateType"

    comment: list[str] = field(
        default_factory=list,
        metadata={
            "name": "COMMENT",
            "type": "Element",
            "namespace": "",
        },
    )
    ref_frame_a: str = field(
        metadata={
            "name": "REF_FRAME_A",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    ref_frame_b: str = field(
        metadata={
            "name": "REF_FRAME_B",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    euler_rot_seq: RotseqType = field(
        metadata={
            "name": "EULER_ROT_SEQ",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    angle_1: AngleType = field(
        metadata={
            "name": "ANGLE_1",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    angle_2: AngleType = field(
        metadata={
            "name": "ANGLE_2",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    angle_3: AngleType = field(
        metadata={
            "name": "ANGLE_3",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    angle_1_dot: None | AngleRateType = field(
        default=None,
        metadata={
            "name": "ANGLE_1_DOT",
            "type": "Element",
            "namespace": "",
        },
    )
    angle_2_dot: None | AngleRateType = field(
        default=None,
        metadata={
            "name": "ANGLE_2_DOT",
            "type": "Element",
            "namespace": "",
        },
    )
    angle_3_dot: None | AngleRateType = field(
        default=None,
        metadata={
            "name": "ANGLE_3_DOT",
            "type": "Element",
            "namespace": "",
        },
    )


@dataclass(kw_only=True)
class EulerAngleType:
    class Meta:
        name = "eulerAngleType"

    epoch: str = field(
        metadata={
            "name": "EPOCH",
            "type": "Element",
            "namespace": "",
            "required": True,
            "pattern": r"\-?\d{4}\d*-((\d{2}\-\d{2})|\d{3})T\d{2}:\d{2}:\d{2}(\.\d*)?(Z|[+|\-]\d{2}:\d{2})?|[+|\-]?\d*(\.\d*)?",
        }
    )
    angle_1: AngleType = field(
        metadata={
            "name": "ANGLE_1",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    angle_2: AngleType = field(
        metadata={
            "name": "ANGLE_2",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    angle_3: AngleType = field(
        metadata={
            "name": "ANGLE_3",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class GroundImpactParametersType:
    class Meta:
        name = "groundImpactParametersType"

    comment: list[str] = field(
        default_factory=list,
        metadata={
            "name": "COMMENT",
            "type": "Element",
            "namespace": "",
        },
    )
    probability_of_impact: None | float = field(
        default=None,
        metadata={
            "name": "PROBABILITY_OF_IMPACT",
            "type": "Element",
            "namespace": "",
            "min_inclusive": 0.0,
            "max_inclusive": 1.0,
        },
    )
    probability_of_burn_up: None | float = field(
        default=None,
        metadata={
            "name": "PROBABILITY_OF_BURN_UP",
            "type": "Element",
            "namespace": "",
            "min_inclusive": 0.0,
            "max_inclusive": 1.0,
        },
    )
    probability_of_break_up: None | float = field(
        default=None,
        metadata={
            "name": "PROBABILITY_OF_BREAK_UP",
            "type": "Element",
            "namespace": "",
            "min_inclusive": 0.0,
            "max_inclusive": 1.0,
        },
    )
    probability_of_land_impact: None | float = field(
        default=None,
        metadata={
            "name": "PROBABILITY_OF_LAND_IMPACT",
            "type": "Element",
            "namespace": "",
            "min_inclusive": 0.0,
            "max_inclusive": 1.0,
        },
    )
    probability_of_casualty: None | float = field(
        default=None,
        metadata={
            "name": "PROBABILITY_OF_CASUALTY",
            "type": "Element",
            "namespace": "",
            "min_inclusive": 0.0,
            "max_inclusive": 1.0,
        },
    )
    nominal_impact_epoch: None | str = field(
        default=None,
        metadata={
            "name": "NOMINAL_IMPACT_EPOCH",
            "type": "Element",
            "namespace": "",
            "pattern": r"\-?\d{4}\d*-((\d{2}\-\d{2})|\d{3})T\d{2}:\d{2}:\d{2}(\.\d*)?(Z|[+|\-]\d{2}:\d{2})?|[+|\-]?\d*(\.\d*)?",
        },
    )
    impact_window_start: None | str = field(
        default=None,
        metadata={
            "name": "IMPACT_WINDOW_START",
            "type": "Element",
            "namespace": "",
            "pattern": r"\-?\d{4}\d*-((\d{2}\-\d{2})|\d{3})T\d{2}:\d{2}:\d{2}(\.\d*)?(Z|[+|\-]\d{2}:\d{2})?|[+|\-]?\d*(\.\d*)?",
        },
    )
    impact_window_end: None | str = field(
        default=None,
        metadata={
            "name": "IMPACT_WINDOW_END",
            "type": "Element",
            "namespace": "",
            "pattern": r"\-?\d{4}\d*-((\d{2}\-\d{2})|\d{3})T\d{2}:\d{2}:\d{2}(\.\d*)?(Z|[+|\-]\d{2}:\d{2})?|[+|\-]?\d*(\.\d*)?",
        },
    )
    impact_ref_frame: None | str = field(
        default=None,
        metadata={
            "name": "IMPACT_REF_FRAME",
            "type": "Element",
            "namespace": "",
        },
    )
    nominal_impact_lon: None | LonType = field(
        default=None,
        metadata={
            "name": "NOMINAL_IMPACT_LON",
            "type": "Element",
            "namespace": "",
        },
    )
    nominal_impact_lat: None | LatType = field(
        default=None,
        metadata={
            "name": "NOMINAL_IMPACT_LAT",
            "type": "Element",
            "namespace": "",
        },
    )
    nominal_impact_alt: None | AltType = field(
        default=None,
        metadata={
            "name": "NOMINAL_IMPACT_ALT",
            "type": "Element",
            "namespace": "",
        },
    )
    impact_1_confidence: None | PercentageTypeUr = field(
        default=None,
        metadata={
            "name": "IMPACT_1_CONFIDENCE",
            "type": "Element",
            "namespace": "",
        },
    )
    impact_1_start_lon: None | LonType = field(
        default=None,
        metadata={
            "name": "IMPACT_1_START_LON",
            "type": "Element",
            "namespace": "",
        },
    )
    impact_1_start_lat: None | LatType = field(
        default=None,
        metadata={
            "name": "IMPACT_1_START_LAT",
            "type": "Element",
            "namespace": "",
        },
    )
    impact_1_stop_lon: None | LonType = field(
        default=None,
        metadata={
            "name": "IMPACT_1_STOP_LON",
            "type": "Element",
            "namespace": "",
        },
    )
    impact_1_stop_lat: None | LatType = field(
        default=None,
        metadata={
            "name": "IMPACT_1_STOP_LAT",
            "type": "Element",
            "namespace": "",
        },
    )
    impact_1_cross_track: None | DistanceType = field(
        default=None,
        metadata={
            "name": "IMPACT_1_CROSS_TRACK",
            "type": "Element",
            "namespace": "",
        },
    )
    impact_2_confidence: None | PercentageTypeUr = field(
        default=None,
        metadata={
            "name": "IMPACT_2_CONFIDENCE",
            "type": "Element",
            "namespace": "",
        },
    )
    impact_2_start_lon: None | LonType = field(
        default=None,
        metadata={
            "name": "IMPACT_2_START_LON",
            "type": "Element",
            "namespace": "",
        },
    )
    impact_2_start_lat: None | LatType = field(
        default=None,
        metadata={
            "name": "IMPACT_2_START_LAT",
            "type": "Element",
            "namespace": "",
        },
    )
    impact_2_stop_lon: None | LonType = field(
        default=None,
        metadata={
            "name": "IMPACT_2_STOP_LON",
            "type": "Element",
            "namespace": "",
        },
    )
    impact_2_stop_lat: None | LatType = field(
        default=None,
        metadata={
            "name": "IMPACT_2_STOP_LAT",
            "type": "Element",
            "namespace": "",
        },
    )
    impact_2_cross_track: None | DistanceType = field(
        default=None,
        metadata={
            "name": "IMPACT_2_CROSS_TRACK",
            "type": "Element",
            "namespace": "",
        },
    )
    impact_3_confidence: None | PercentageTypeUr = field(
        default=None,
        metadata={
            "name": "IMPACT_3_CONFIDENCE",
            "type": "Element",
            "namespace": "",
        },
    )
    impact_3_start_lon: None | LonType = field(
        default=None,
        metadata={
            "name": "IMPACT_3_START_LON",
            "type": "Element",
            "namespace": "",
        },
    )
    impact_3_start_lat: None | LatType = field(
        default=None,
        metadata={
            "name": "IMPACT_3_START_LAT",
            "type": "Element",
            "namespace": "",
        },
    )
    impact_3_stop_lon: None | LonType = field(
        default=None,
        metadata={
            "name": "IMPACT_3_STOP_LON",
            "type": "Element",
            "namespace": "",
        },
    )
    impact_3_stop_lat: None | LatType = field(
        default=None,
        metadata={
            "name": "IMPACT_3_STOP_LAT",
            "type": "Element",
            "namespace": "",
        },
    )
    impact_3_cross_track: None | DistanceType = field(
        default=None,
        metadata={
            "name": "IMPACT_3_CROSS_TRACK",
            "type": "Element",
            "namespace": "",
        },
    )


@dataclass(kw_only=True)
class InertiaStateType:
    class Meta:
        name = "inertiaStateType"

    comment: list[str] = field(
        default_factory=list,
        metadata={
            "name": "COMMENT",
            "type": "Element",
            "namespace": "",
        },
    )
    inertia_ref_frame: str = field(
        metadata={
            "name": "INERTIA_REF_FRAME",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    ixx: MomentType = field(
        metadata={
            "name": "IXX",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    iyy: MomentType = field(
        metadata={
            "name": "IYY",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    izz: MomentType = field(
        metadata={
            "name": "IZZ",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    ixy: MomentType = field(
        metadata={
            "name": "IXY",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    ixz: MomentType = field(
        metadata={
            "name": "IXZ",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    iyz: MomentType = field(
        metadata={
            "name": "IYZ",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class OdParametersType:
    class Meta:
        name = "odParametersType"

    comment: list[str] = field(
        default_factory=list,
        metadata={
            "name": "COMMENT",
            "type": "Element",
            "namespace": "",
        },
    )
    time_lastob_start: None | str = field(
        default=None,
        metadata={
            "name": "TIME_LASTOB_START",
            "type": "Element",
            "namespace": "",
            "pattern": r"\-?\d{4}\d*-((\d{2}\-\d{2})|\d{3})T\d{2}:\d{2}:\d{2}(\.\d*)?(Z|[+|\-]\d{2}:\d{2})?|[+|\-]?\d*(\.\d*)?",
        },
    )
    time_lastob_end: None | str = field(
        default=None,
        metadata={
            "name": "TIME_LASTOB_END",
            "type": "Element",
            "namespace": "",
            "pattern": r"\-?\d{4}\d*-((\d{2}\-\d{2})|\d{3})T\d{2}:\d{2}:\d{2}(\.\d*)?(Z|[+|\-]\d{2}:\d{2})?|[+|\-]?\d*(\.\d*)?",
        },
    )
    recommended_od_span: None | DayIntervalTypeUo = field(
        default=None,
        metadata={
            "name": "RECOMMENDED_OD_SPAN",
            "type": "Element",
            "namespace": "",
        },
    )
    actual_od_span: None | DayIntervalTypeUo = field(
        default=None,
        metadata={
            "name": "ACTUAL_OD_SPAN",
            "type": "Element",
            "namespace": "",
        },
    )
    obs_available: None | int = field(
        default=None,
        metadata={
            "name": "OBS_AVAILABLE",
            "type": "Element",
            "namespace": "",
        },
    )
    obs_used: None | int = field(
        default=None,
        metadata={
            "name": "OBS_USED",
            "type": "Element",
            "namespace": "",
        },
    )
    tracks_available: None | int = field(
        default=None,
        metadata={
            "name": "TRACKS_AVAILABLE",
            "type": "Element",
            "namespace": "",
        },
    )
    tracks_used: None | int = field(
        default=None,
        metadata={
            "name": "TRACKS_USED",
            "type": "Element",
            "namespace": "",
        },
    )
    residuals_accepted: None | PercentageTypeUo = field(
        default=None,
        metadata={
            "name": "RESIDUALS_ACCEPTED",
            "type": "Element",
            "namespace": "",
        },
    )
    weighted_rms: None | float = field(
        default=None,
        metadata={
            "name": "WEIGHTED_RMS",
            "type": "Element",
            "namespace": "",
            "min_inclusive": 0.0,
        },
    )


@dataclass(kw_only=True)
class OemCovarianceMatrixType(OemCovarianceMatrixAbstractType):
    class Meta:
        name = "oemCovarianceMatrixType"

    cx_x: PositionCovarianceType = field(
        metadata={
            "name": "CX_X",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    cy_x: PositionCovarianceType = field(
        metadata={
            "name": "CY_X",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    cy_y: PositionCovarianceType = field(
        metadata={
            "name": "CY_Y",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    cz_x: PositionCovarianceType = field(
        metadata={
            "name": "CZ_X",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    cz_y: PositionCovarianceType = field(
        metadata={
            "name": "CZ_Y",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    cz_z: PositionCovarianceType = field(
        metadata={
            "name": "CZ_Z",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    cx_dot_x: PositionVelocityCovarianceType = field(
        metadata={
            "name": "CX_DOT_X",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    cx_dot_y: PositionVelocityCovarianceType = field(
        metadata={
            "name": "CX_DOT_Y",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    cx_dot_z: PositionVelocityCovarianceType = field(
        metadata={
            "name": "CX_DOT_Z",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    cx_dot_x_dot: VelocityCovarianceType = field(
        metadata={
            "name": "CX_DOT_X_DOT",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    cy_dot_x: PositionVelocityCovarianceType = field(
        metadata={
            "name": "CY_DOT_X",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    cy_dot_y: PositionVelocityCovarianceType = field(
        metadata={
            "name": "CY_DOT_Y",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    cy_dot_z: PositionVelocityCovarianceType = field(
        metadata={
            "name": "CY_DOT_Z",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    cy_dot_x_dot: VelocityCovarianceType = field(
        metadata={
            "name": "CY_DOT_X_DOT",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    cy_dot_y_dot: VelocityCovarianceType = field(
        metadata={
            "name": "CY_DOT_Y_DOT",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    cz_dot_x: PositionVelocityCovarianceType = field(
        metadata={
            "name": "CZ_DOT_X",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    cz_dot_y: PositionVelocityCovarianceType = field(
        metadata={
            "name": "CZ_DOT_Y",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    cz_dot_z: PositionVelocityCovarianceType = field(
        metadata={
            "name": "CZ_DOT_Z",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    cz_dot_x_dot: VelocityCovarianceType = field(
        metadata={
            "name": "CZ_DOT_X_DOT",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    cz_dot_y_dot: VelocityCovarianceType = field(
        metadata={
            "name": "CZ_DOT_Y_DOT",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    cz_dot_z_dot: VelocityCovarianceType = field(
        metadata={
            "name": "CZ_DOT_Z_DOT",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class OpmCovarianceMatrixType(OpmCovarianceMatrixAbstractType):
    class Meta:
        name = "opmCovarianceMatrixType"

    cx_x: PositionCovarianceType = field(
        metadata={
            "name": "CX_X",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    cy_x: PositionCovarianceType = field(
        metadata={
            "name": "CY_X",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    cy_y: PositionCovarianceType = field(
        metadata={
            "name": "CY_Y",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    cz_x: PositionCovarianceType = field(
        metadata={
            "name": "CZ_X",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    cz_y: PositionCovarianceType = field(
        metadata={
            "name": "CZ_Y",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    cz_z: PositionCovarianceType = field(
        metadata={
            "name": "CZ_Z",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    cx_dot_x: PositionVelocityCovarianceType = field(
        metadata={
            "name": "CX_DOT_X",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    cx_dot_y: PositionVelocityCovarianceType = field(
        metadata={
            "name": "CX_DOT_Y",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    cx_dot_z: PositionVelocityCovarianceType = field(
        metadata={
            "name": "CX_DOT_Z",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    cx_dot_x_dot: VelocityCovarianceType = field(
        metadata={
            "name": "CX_DOT_X_DOT",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    cy_dot_x: PositionVelocityCovarianceType = field(
        metadata={
            "name": "CY_DOT_X",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    cy_dot_y: PositionVelocityCovarianceType = field(
        metadata={
            "name": "CY_DOT_Y",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    cy_dot_z: PositionVelocityCovarianceType = field(
        metadata={
            "name": "CY_DOT_Z",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    cy_dot_x_dot: VelocityCovarianceType = field(
        metadata={
            "name": "CY_DOT_X_DOT",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    cy_dot_y_dot: VelocityCovarianceType = field(
        metadata={
            "name": "CY_DOT_Y_DOT",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    cz_dot_x: PositionVelocityCovarianceType = field(
        metadata={
            "name": "CZ_DOT_X",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    cz_dot_y: PositionVelocityCovarianceType = field(
        metadata={
            "name": "CZ_DOT_Y",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    cz_dot_z: PositionVelocityCovarianceType = field(
        metadata={
            "name": "CZ_DOT_Z",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    cz_dot_x_dot: VelocityCovarianceType = field(
        metadata={
            "name": "CZ_DOT_X_DOT",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    cz_dot_y_dot: VelocityCovarianceType = field(
        metadata={
            "name": "CZ_DOT_Y_DOT",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    cz_dot_z_dot: VelocityCovarianceType = field(
        metadata={
            "name": "CZ_DOT_Z_DOT",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class QuaternionDotType:
    class Meta:
        name = "quaternionDotType"

    q1_dot: QuaternionDotComponentType = field(
        metadata={
            "name": "Q1_DOT",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    q2_dot: QuaternionDotComponentType = field(
        metadata={
            "name": "Q2_DOT",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    q3_dot: QuaternionDotComponentType = field(
        metadata={
            "name": "Q3_DOT",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    qc_dot: QuaternionDotComponentType = field(
        metadata={
            "name": "QC_DOT",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class RdmSpacecraftParametersType:
    class Meta:
        name = "rdmSpacecraftParametersType"

    comment: list[str] = field(
        default_factory=list,
        metadata={
            "name": "COMMENT",
            "type": "Element",
            "namespace": "",
        },
    )
    wet_mass: None | MassType = field(
        default=None,
        metadata={
            "name": "WET_MASS",
            "type": "Element",
            "namespace": "",
        },
    )
    dry_mass: None | MassType = field(
        default=None,
        metadata={
            "name": "DRY_MASS",
            "type": "Element",
            "namespace": "",
        },
    )
    hazardous_substances: None | str = field(
        default=None,
        metadata={
            "name": "HAZARDOUS_SUBSTANCES",
            "type": "Element",
            "namespace": "",
        },
    )
    solar_rad_area: None | AreaType = field(
        default=None,
        metadata={
            "name": "SOLAR_RAD_AREA",
            "type": "Element",
            "namespace": "",
        },
    )
    solar_rad_coeff: None | float = field(
        default=None,
        metadata={
            "name": "SOLAR_RAD_COEFF",
            "type": "Element",
            "namespace": "",
            "min_inclusive": 0.0,
        },
    )
    drag_area: None | AreaType = field(
        default=None,
        metadata={
            "name": "DRAG_AREA",
            "type": "Element",
            "namespace": "",
        },
    )
    drag_coeff: None | float = field(
        default=None,
        metadata={
            "name": "DRAG_COEFF",
            "type": "Element",
            "namespace": "",
            "min_inclusive": 0.0,
        },
    )
    rcs: None | AreaType = field(
        default=None,
        metadata={
            "name": "RCS",
            "type": "Element",
            "namespace": "",
        },
    )
    ballistic_coeff: None | BallisticCoeffType = field(
        default=None,
        metadata={
            "name": "BALLISTIC_COEFF",
            "type": "Element",
            "namespace": "",
        },
    )
    thrust_acceleration: None | Ms2Type = field(
        default=None,
        metadata={
            "name": "THRUST_ACCELERATION",
            "type": "Element",
            "namespace": "",
        },
    )


@dataclass(kw_only=True)
class SpacecraftParametersType:
    class Meta:
        name = "spacecraftParametersType"

    comment: list[str] = field(
        default_factory=list,
        metadata={
            "name": "COMMENT",
            "type": "Element",
            "namespace": "",
        },
    )
    mass: None | MassType = field(
        default=None,
        metadata={
            "name": "MASS",
            "type": "Element",
            "namespace": "",
        },
    )
    solar_rad_area: None | AreaType = field(
        default=None,
        metadata={
            "name": "SOLAR_RAD_AREA",
            "type": "Element",
            "namespace": "",
        },
    )
    solar_rad_coeff: None | float = field(
        default=None,
        metadata={
            "name": "SOLAR_RAD_COEFF",
            "type": "Element",
            "namespace": "",
            "min_inclusive": 0.0,
        },
    )
    drag_area: None | AreaType = field(
        default=None,
        metadata={
            "name": "DRAG_AREA",
            "type": "Element",
            "namespace": "",
        },
    )
    drag_coeff: None | float = field(
        default=None,
        metadata={
            "name": "DRAG_COEFF",
            "type": "Element",
            "namespace": "",
            "min_inclusive": 0.0,
        },
    )


@dataclass(kw_only=True)
class SpinNutationMomType:
    class Meta:
        name = "spinNutationMomType"

    epoch: str = field(
        metadata={
            "name": "EPOCH",
            "type": "Element",
            "namespace": "",
            "required": True,
            "pattern": r"\-?\d{4}\d*-((\d{2}\-\d{2})|\d{3})T\d{2}:\d{2}:\d{2}(\.\d*)?(Z|[+|\-]\d{2}:\d{2})?|[+|\-]?\d*(\.\d*)?",
        }
    )
    spin_alpha: AngleType = field(
        metadata={
            "name": "SPIN_ALPHA",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    spin_delta: AngleType = field(
        metadata={
            "name": "SPIN_DELTA",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    spin_angle: AngleType = field(
        metadata={
            "name": "SPIN_ANGLE",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    spin_angle_vel: AngleRateType = field(
        metadata={
            "name": "SPIN_ANGLE_VEL",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    momentum_alpha: AngleType = field(
        metadata={
            "name": "MOMENTUM_ALPHA",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    momentum_delta: AngleType = field(
        metadata={
            "name": "MOMENTUM_DELTA",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    nutation_vel: AngleRateType = field(
        metadata={
            "name": "NUTATION_VEL",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class SpinNutationType:
    class Meta:
        name = "spinNutationType"

    epoch: str = field(
        metadata={
            "name": "EPOCH",
            "type": "Element",
            "namespace": "",
            "required": True,
            "pattern": r"\-?\d{4}\d*-((\d{2}\-\d{2})|\d{3})T\d{2}:\d{2}:\d{2}(\.\d*)?(Z|[+|\-]\d{2}:\d{2})?|[+|\-]?\d*(\.\d*)?",
        }
    )
    spin_alpha: AngleType = field(
        metadata={
            "name": "SPIN_ALPHA",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    spin_delta: AngleType = field(
        metadata={
            "name": "SPIN_DELTA",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    spin_angle: AngleType = field(
        metadata={
            "name": "SPIN_ANGLE",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    spin_angle_vel: AngleRateType = field(
        metadata={
            "name": "SPIN_ANGLE_VEL",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    nutation: AngleType = field(
        metadata={
            "name": "NUTATION",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    nutation_per: DurationType = field(
        metadata={
            "name": "NUTATION_PER",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    nutation_phase: AngleType = field(
        metadata={
            "name": "NUTATION_PHASE",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class SpinStateType:
    class Meta:
        name = "spinStateType"

    comment: list[str] = field(
        default_factory=list,
        metadata={
            "name": "COMMENT",
            "type": "Element",
            "namespace": "",
        },
    )
    ref_frame_a: str = field(
        metadata={
            "name": "REF_FRAME_A",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    ref_frame_b: str = field(
        metadata={
            "name": "REF_FRAME_B",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    spin_alpha: AngleType = field(
        metadata={
            "name": "SPIN_ALPHA",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    spin_delta: AngleType = field(
        metadata={
            "name": "SPIN_DELTA",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    spin_angle: AngleType = field(
        metadata={
            "name": "SPIN_ANGLE",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    spin_angle_vel: AngleRateType = field(
        metadata={
            "name": "SPIN_ANGLE_VEL",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    nutation: None | AngleType = field(
        default=None,
        metadata={
            "name": "NUTATION",
            "type": "Element",
            "namespace": "",
        },
    )
    nutation_per: None | DurationType = field(
        default=None,
        metadata={
            "name": "NUTATION_PER",
            "type": "Element",
            "namespace": "",
        },
    )
    nutation_phase: None | AngleType = field(
        default=None,
        metadata={
            "name": "NUTATION_PHASE",
            "type": "Element",
            "namespace": "",
        },
    )
    momentum_alpha: None | AngleType = field(
        default=None,
        metadata={
            "name": "MOMENTUM_ALPHA",
            "type": "Element",
            "namespace": "",
        },
    )
    momentum_delta: None | AngleType = field(
        default=None,
        metadata={
            "name": "MOMENTUM_DELTA",
            "type": "Element",
            "namespace": "",
        },
    )
    nutation_vel: None | AngleRateType = field(
        default=None,
        metadata={
            "name": "NUTATION_VEL",
            "type": "Element",
            "namespace": "",
        },
    )


@dataclass(kw_only=True)
class SpinType:
    class Meta:
        name = "spinType"

    epoch: str = field(
        metadata={
            "name": "EPOCH",
            "type": "Element",
            "namespace": "",
            "required": True,
            "pattern": r"\-?\d{4}\d*-((\d{2}\-\d{2})|\d{3})T\d{2}:\d{2}:\d{2}(\.\d*)?(Z|[+|\-]\d{2}:\d{2})?|[+|\-]?\d*(\.\d*)?",
        }
    )
    spin_alpha: AngleType = field(
        metadata={
            "name": "SPIN_ALPHA",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    spin_delta: AngleType = field(
        metadata={
            "name": "SPIN_DELTA",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    spin_angle: AngleType = field(
        metadata={
            "name": "SPIN_ANGLE",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    spin_angle_vel: AngleRateType = field(
        metadata={
            "name": "SPIN_ANGLE_VEL",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class StateVectorAccType:
    class Meta:
        name = "stateVectorAccType"

    epoch: str = field(
        metadata={
            "name": "EPOCH",
            "type": "Element",
            "namespace": "",
            "required": True,
            "pattern": r"\-?\d{4}\d*-((\d{2}\-\d{2})|\d{3})T\d{2}:\d{2}:\d{2}(\.\d*)?(Z|[+|\-]\d{2}:\d{2})?|[+|\-]?\d*(\.\d*)?",
        }
    )
    x: PositionTypeUo = field(
        metadata={
            "name": "X",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    y: PositionTypeUo = field(
        metadata={
            "name": "Y",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    z: PositionTypeUo = field(
        metadata={
            "name": "Z",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    x_dot: VelocityTypeUo = field(
        metadata={
            "name": "X_DOT",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    y_dot: VelocityTypeUo = field(
        metadata={
            "name": "Y_DOT",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    z_dot: VelocityTypeUo = field(
        metadata={
            "name": "Z_DOT",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    x_ddot: None | AccType = field(
        default=None,
        metadata={
            "name": "X_DDOT",
            "type": "Element",
            "namespace": "",
        },
    )
    y_ddot: None | AccType = field(
        default=None,
        metadata={
            "name": "Y_DDOT",
            "type": "Element",
            "namespace": "",
        },
    )
    z_ddot: None | AccType = field(
        default=None,
        metadata={
            "name": "Z_DDOT",
            "type": "Element",
            "namespace": "",
        },
    )


@dataclass(kw_only=True)
class StateVectorType:
    class Meta:
        name = "stateVectorType"

    comment: list[str] = field(
        default_factory=list,
        metadata={
            "name": "COMMENT",
            "type": "Element",
            "namespace": "",
        },
    )
    epoch: str = field(
        metadata={
            "name": "EPOCH",
            "type": "Element",
            "namespace": "",
            "required": True,
            "pattern": r"\-?\d{4}\d*-((\d{2}\-\d{2})|\d{3})T\d{2}:\d{2}:\d{2}(\.\d*)?(Z|[+|\-]\d{2}:\d{2})?|[+|\-]?\d*(\.\d*)?",
        }
    )
    x: PositionTypeUo = field(
        metadata={
            "name": "X",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    y: PositionTypeUo = field(
        metadata={
            "name": "Y",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    z: PositionTypeUo = field(
        metadata={
            "name": "Z",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    x_dot: VelocityTypeUo = field(
        metadata={
            "name": "X_DOT",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    y_dot: VelocityTypeUo = field(
        metadata={
            "name": "Y_DOT",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    z_dot: VelocityTypeUo = field(
        metadata={
            "name": "Z_DOT",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class QuaternionAngVelType:
    class Meta:
        name = "quaternionAngVelType"

    epoch: str = field(
        metadata={
            "name": "EPOCH",
            "type": "Element",
            "namespace": "",
            "required": True,
            "pattern": r"\-?\d{4}\d*-((\d{2}\-\d{2})|\d{3})T\d{2}:\d{2}:\d{2}(\.\d*)?(Z|[+|\-]\d{2}:\d{2})?|[+|\-]?\d*(\.\d*)?",
        }
    )
    quaternion: QuaternionType = field(
        metadata={
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    ang_vel: AngVelType = field(
        metadata={
            "name": "angVel",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class QuaternionDerivativeType:
    class Meta:
        name = "quaternionDerivativeType"

    epoch: str = field(
        metadata={
            "name": "EPOCH",
            "type": "Element",
            "namespace": "",
            "required": True,
            "pattern": r"\-?\d{4}\d*-((\d{2}\-\d{2})|\d{3})T\d{2}:\d{2}:\d{2}(\.\d*)?(Z|[+|\-]\d{2}:\d{2})?|[+|\-]?\d*(\.\d*)?",
        }
    )
    quaternion: QuaternionType = field(
        metadata={
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    quaternion_dot: QuaternionDotType = field(
        metadata={
            "name": "quaternionDot",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class QuaternionStateType:
    class Meta:
        name = "quaternionStateType"

    comment: list[str] = field(
        default_factory=list,
        metadata={
            "name": "COMMENT",
            "type": "Element",
            "namespace": "",
        },
    )
    ref_frame_a: str = field(
        metadata={
            "name": "REF_FRAME_A",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    ref_frame_b: str = field(
        metadata={
            "name": "REF_FRAME_B",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    quaternion: QuaternionType = field(
        metadata={
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    quaternion_dot: None | QuaternionDotType = field(
        default=None,
        metadata={
            "name": "quaternionDot",
            "type": "Element",
            "namespace": "",
        },
    )


@dataclass(kw_only=True)
class AttitudeStateType:
    class Meta:
        name = "attitudeStateType"

    quaternion_ephemeris: None | QuaternionEphemerisType = field(
        default=None,
        metadata={
            "name": "quaternionEphemeris",
            "type": "Element",
            "namespace": "",
        },
    )
    quaternion_derivative: None | QuaternionDerivativeType = field(
        default=None,
        metadata={
            "name": "quaternionDerivative",
            "type": "Element",
            "namespace": "",
        },
    )
    quaternion_ang_vel: None | QuaternionAngVelType = field(
        default=None,
        metadata={
            "name": "quaternionAngVel",
            "type": "Element",
            "namespace": "",
        },
    )
    euler_angle: None | EulerAngleType = field(
        default=None,
        metadata={
            "name": "eulerAngle",
            "type": "Element",
            "namespace": "",
        },
    )
    euler_angle_derivative: None | EulerAngleDerivativeType = field(
        default=None,
        metadata={
            "name": "eulerAngleDerivative",
            "type": "Element",
            "namespace": "",
        },
    )
    euler_angle_ang_vel: None | EulerAngleAngVelType = field(
        default=None,
        metadata={
            "name": "eulerAngleAngVel",
            "type": "Element",
            "namespace": "",
        },
    )
    spin: None | SpinType = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "",
        },
    )
    spin_nutation: None | SpinNutationType = field(
        default=None,
        metadata={
            "name": "spinNutation",
            "type": "Element",
            "namespace": "",
        },
    )
    spin_nutation_mom: None | SpinNutationMomType = field(
        default=None,
        metadata={
            "name": "spinNutationMom",
            "type": "Element",
            "namespace": "",
        },
    )



"""This file was generated by xsdata, v26.1, on 2026-02-18 00:41:07

Generator: DataclassGenerator
See: https://xsdata.readthedocs.io/
"""

from __future__ import annotations

from dataclasses import dataclass, field

from ccsds_ndm.models.ndmxml4.ndmxml_4_0_0_common_4_0 import (
    AcmAttitudeType,
    AcmCovarianceLineType,
    AdMethodType,
    AdmHeader,
    AngleRateType,
    AttBasisType,
    AttRateType,
    CpType,
    DurationType,
    FrequencyType,
    MassType,
    MomentType,
    RelTimeType,
    RotseqType,
    SensorNoiseType,
    SigmaUtype,
    SigmaVtype,
    TargetMomentumType,
    TimeOffsetType,
    UserDefinedType,
)

__NAMESPACE__ = "urn:ccsds:schema:ndmxml"


@dataclass(kw_only=True)
class AcmAttitudeStateType:
    class Meta:
        name = "acmAttitudeStateType"

    comment: list[str] = field(
        default_factory=list,
        metadata={
            "name": "COMMENT",
            "type": "Element",
            "namespace": "",
        },
    )
    att_id: None | str = field(
        default=None,
        metadata={
            "name": "ATT_ID",
            "type": "Element",
            "namespace": "",
        },
    )
    att_prev_id: None | str = field(
        default=None,
        metadata={
            "name": "ATT_PREV_ID",
            "type": "Element",
            "namespace": "",
        },
    )
    att_basis: None | AttBasisType = field(
        default=None,
        metadata={
            "name": "ATT_BASIS",
            "type": "Element",
            "namespace": "",
        },
    )
    att_basis_id: None | str = field(
        default=None,
        metadata={
            "name": "ATT_BASIS_ID",
            "type": "Element",
            "namespace": "",
        },
    )
    ref_frame_a: str = field(
        metadata={
            "name": "REF_FRAME_A",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    ref_frame_b: str = field(
        metadata={
            "name": "REF_FRAME_B",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    number_states: int = field(
        metadata={
            "name": "NUMBER_STATES",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    att_type: AcmAttitudeType = field(
        metadata={
            "name": "ATT_TYPE",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    euler_rot_seq: None | RotseqType = field(
        default=None,
        metadata={
            "name": "EULER_ROT_SEQ",
            "type": "Element",
            "namespace": "",
        },
    )
    rate_type: None | AttRateType = field(
        default=None,
        metadata={
            "name": "RATE_TYPE",
            "type": "Element",
            "namespace": "",
        },
    )
    att_line: list[str] = field(
        default_factory=list,
        metadata={
            "name": "attLine",
            "type": "Element",
            "namespace": "",
            "min_occurs": 1,
        },
    )


@dataclass(kw_only=True)
class AcmCovarianceMatrixType:
    class Meta:
        name = "acmCovarianceMatrixType"

    comment: list[str] = field(
        default_factory=list,
        metadata={
            "name": "COMMENT",
            "type": "Element",
            "namespace": "",
        },
    )
    cov_id: None | str = field(
        default=None,
        metadata={
            "name": "COV_ID",
            "type": "Element",
            "namespace": "",
        },
    )
    cov_prev_id: None | str = field(
        default=None,
        metadata={
            "name": "COV_PREV_ID",
            "type": "Element",
            "namespace": "",
        },
    )
    cov_basis: None | AttBasisType = field(
        default=None,
        metadata={
            "name": "COV_BASIS",
            "type": "Element",
            "namespace": "",
        },
    )
    cov_basis_id: None | str = field(
        default=None,
        metadata={
            "name": "COV_BASIS_ID",
            "type": "Element",
            "namespace": "",
        },
    )
    cov_ref_frame: None | str = field(
        default=None,
        metadata={
            "name": "COV_REF_FRAME",
            "type": "Element",
            "namespace": "",
        },
    )
    cov_type: AcmCovarianceLineType = field(
        metadata={
            "name": "COV_TYPE",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    cov_line: list[str] = field(
        default_factory=list,
        metadata={
            "name": "covLine",
            "type": "Element",
            "namespace": "",
            "min_occurs": 1,
        },
    )


@dataclass(kw_only=True)
class AcmManeuverParametersType:
    class Meta:
        name = "acmManeuverParametersType"

    comment: list[str] = field(
        default_factory=list,
        metadata={
            "name": "COMMENT",
            "type": "Element",
            "namespace": "",
        },
    )
    man_id: None | str = field(
        default=None,
        metadata={
            "name": "MAN_ID",
            "type": "Element",
            "namespace": "",
        },
    )
    man_prev_id: None | str = field(
        default=None,
        metadata={
            "name": "MAN_PREV_ID",
            "type": "Element",
            "namespace": "",
        },
    )
    man_purpose: str = field(
        metadata={
            "name": "MAN_PURPOSE",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    man_begin_time: RelTimeType = field(
        metadata={
            "name": "MAN_BEGIN_TIME",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    man_end_time: None | RelTimeType = field(
        default=None,
        metadata={
            "name": "MAN_END_TIME",
            "type": "Element",
            "namespace": "",
        },
    )
    man_duration: None | DurationType = field(
        default=None,
        metadata={
            "name": "MAN_DURATION",
            "type": "Element",
            "namespace": "",
        },
    )
    actuator_used: None | str = field(
        default=None,
        metadata={
            "name": "ACTUATOR_USED",
            "type": "Element",
            "namespace": "",
        },
    )
    target_momentum: None | TargetMomentumType = field(
        default=None,
        metadata={
            "name": "TARGET_MOMENTUM",
            "type": "Element",
            "namespace": "",
        },
    )
    target_mom_frame: None | str = field(
        default=None,
        metadata={
            "name": "TARGET_MOM_FRAME",
            "type": "Element",
            "namespace": "",
        },
    )
    target_attitude: list[float] = field(
        default_factory=list,
        metadata={
            "name": "TARGET_ATTITUDE",
            "type": "Element",
            "namespace": "",
            "length": 4,
            "tokens": True,
        },
    )
    target_spinrate: None | AngleRateType = field(
        default=None,
        metadata={
            "name": "TARGET_SPINRATE",
            "type": "Element",
            "namespace": "",
        },
    )


@dataclass(kw_only=True)
class AcmMetadata:
    class Meta:
        name = "acmMetadata"

    comment: list[str] = field(
        default_factory=list,
        metadata={
            "name": "COMMENT",
            "type": "Element",
            "namespace": "",
        },
    )
    object_name: str = field(
        metadata={
            "name": "OBJECT_NAME",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    international_designator: None | str = field(
        default=None,
        metadata={
            "name": "INTERNATIONAL_DESIGNATOR",
            "type": "Element",
            "namespace": "",
        },
    )
    catalog_name: None | str = field(
        default=None,
        metadata={
            "name": "CATALOG_NAME",
            "type": "Element",
            "namespace": "",
        },
    )
    object_designator: None | str = field(
        default=None,
        metadata={
            "name": "OBJECT_DESIGNATOR",
            "type": "Element",
            "namespace": "",
        },
    )
    originator_poc: None | str = field(
        default=None,
        metadata={
            "name": "ORIGINATOR_POC",
            "type": "Element",
            "namespace": "",
        },
    )
    originator_position: None | str = field(
        default=None,
        metadata={
            "name": "ORIGINATOR_POSITION",
            "type": "Element",
            "namespace": "",
        },
    )
    originator_phone: None | str = field(
        default=None,
        metadata={
            "name": "ORIGINATOR_PHONE",
            "type": "Element",
            "namespace": "",
        },
    )
    originator_email: None | str = field(
        default=None,
        metadata={
            "name": "ORIGINATOR_EMAIL",
            "type": "Element",
            "namespace": "",
        },
    )
    originator_address: None | str = field(
        default=None,
        metadata={
            "name": "ORIGINATOR_ADDRESS",
            "type": "Element",
            "namespace": "",
        },
    )
    odm_msg_link: None | str = field(
        default=None,
        metadata={
            "name": "ODM_MSG_LINK",
            "type": "Element",
            "namespace": "",
        },
    )
    center_name: None | str = field(
        default=None,
        metadata={
            "name": "CENTER_NAME",
            "type": "Element",
            "namespace": "",
        },
    )
    time_system: str = field(
        metadata={
            "name": "TIME_SYSTEM",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    epoch_tzero: str = field(
        metadata={
            "name": "EPOCH_TZERO",
            "type": "Element",
            "namespace": "",
            "required": True,
            "pattern": r"\-?\d{4}\d*-((\d{2}\-\d{2})|\d{3})T\d{2}:\d{2}:\d{2}(\.\d*)?(Z|[+|\-]\d{2}:\d{2})?|[+|\-]?\d*(\.\d*)?",
        }
    )
    acm_data_elements: None | str = field(
        default=None,
        metadata={
            "name": "ACM_DATA_ELEMENTS",
            "type": "Element",
            "namespace": "",
        },
    )
    start_time: None | str = field(
        default=None,
        metadata={
            "name": "START_TIME",
            "type": "Element",
            "namespace": "",
            "pattern": r"\-?\d{4}\d*-((\d{2}\-\d{2})|\d{3})T\d{2}:\d{2}:\d{2}(\.\d*)?(Z|[+|\-]\d{2}:\d{2})?|[+|\-]?\d*(\.\d*)?",
        },
    )
    stop_time: None | str = field(
        default=None,
        metadata={
            "name": "STOP_TIME",
            "type": "Element",
            "namespace": "",
            "pattern": r"\-?\d{4}\d*-((\d{2}\-\d{2})|\d{3})T\d{2}:\d{2}:\d{2}(\.\d*)?(Z|[+|\-]\d{2}:\d{2})?|[+|\-]?\d*(\.\d*)?",
        },
    )
    taimutc_at_tzero: None | TimeOffsetType = field(
        default=None,
        metadata={
            "name": "TAIMUTC_AT_TZERO",
            "type": "Element",
            "namespace": "",
        },
    )
    next_leap_epoch: None | str = field(
        default=None,
        metadata={
            "name": "NEXT_LEAP_EPOCH",
            "type": "Element",
            "namespace": "",
            "pattern": r"\-?\d{4}\d*-((\d{2}\-\d{2})|\d{3})T\d{2}:\d{2}:\d{2}(\.\d*)?(Z|[+|\-]\d{2}:\d{2})?|[+|\-]?\d*(\.\d*)?",
        },
    )
    next_leap_taimutc: None | TimeOffsetType = field(
        default=None,
        metadata={
            "name": "NEXT_LEAP_TAIMUTC",
            "type": "Element",
            "namespace": "",
        },
    )


@dataclass(kw_only=True)
class AcmPhysicalDescriptionType:
    class Meta:
        name = "acmPhysicalDescriptionType"

    comment: list[str] = field(
        default_factory=list,
        metadata={
            "name": "COMMENT",
            "type": "Element",
            "namespace": "",
        },
    )
    drag_coeff: None | float = field(
        default=None,
        metadata={
            "name": "DRAG_COEFF",
            "type": "Element",
            "namespace": "",
        },
    )
    wet_mass: None | MassType = field(
        default=None,
        metadata={
            "name": "WET_MASS",
            "type": "Element",
            "namespace": "",
        },
    )
    dry_mass: None | MassType = field(
        default=None,
        metadata={
            "name": "DRY_MASS",
            "type": "Element",
            "namespace": "",
        },
    )
    cp_ref_frame: None | str = field(
        default=None,
        metadata={
            "name": "CP_REF_FRAME",
            "type": "Element",
            "namespace": "",
        },
    )
    cp: None | CpType = field(
        default=None,
        metadata={
            "name": "CP",
            "type": "Element",
            "namespace": "",
        },
    )
    inertia_ref_frame: None | str = field(
        default=None,
        metadata={
            "name": "INERTIA_REF_FRAME",
            "type": "Element",
            "namespace": "",
        },
    )
    ixx: None | MomentType = field(
        default=None,
        metadata={
            "name": "IXX",
            "type": "Element",
            "namespace": "",
        },
    )
    iyy: None | MomentType = field(
        default=None,
        metadata={
            "name": "IYY",
            "type": "Element",
            "namespace": "",
        },
    )
    izz: None | MomentType = field(
        default=None,
        metadata={
            "name": "IZZ",
            "type": "Element",
            "namespace": "",
        },
    )
    ixy: None | MomentType = field(
        default=None,
        metadata={
            "name": "IXY",
            "type": "Element",
            "namespace": "",
        },
    )
    ixz: None | MomentType = field(
        default=None,
        metadata={
            "name": "IXZ",
            "type": "Element",
            "namespace": "",
        },
    )
    iyz: None | MomentType = field(
        default=None,
        metadata={
            "name": "IYZ",
            "type": "Element",
            "namespace": "",
        },
    )


@dataclass(kw_only=True)
class SensorDataType:
    class Meta:
        name = "sensorDataType"

    sensor_number: None | int = field(
        default=None,
        metadata={
            "name": "SENSOR_NUMBER",
            "type": "Element",
            "namespace": "",
        },
    )
    sensor_used: None | str = field(
        default=None,
        metadata={
            "name": "SENSOR_USED",
            "type": "Element",
            "namespace": "",
        },
    )
    number_sensor_noise_covariance: None | int = field(
        default=None,
        metadata={
            "name": "NUMBER_SENSOR_NOISE_COVARIANCE",
            "type": "Element",
            "namespace": "",
        },
    )
    sensor_noise_stddev: None | SensorNoiseType = field(
        default=None,
        metadata={
            "name": "SENSOR_NOISE_STDDEV",
            "type": "Element",
            "namespace": "",
        },
    )
    sensor_frequency: None | FrequencyType = field(
        default=None,
        metadata={
            "name": "SENSOR_FREQUENCY",
            "type": "Element",
            "namespace": "",
        },
    )


@dataclass(kw_only=True)
class AcmAdParametersType:
    class Meta:
        name = "acmAdParametersType"

    comment: list[str] = field(
        default_factory=list,
        metadata={
            "name": "COMMENT",
            "type": "Element",
            "namespace": "",
        },
    )
    ad_id: None | str = field(
        default=None,
        metadata={
            "name": "AD_ID",
            "type": "Element",
            "namespace": "",
        },
    )
    ad_prev_id: None | str = field(
        default=None,
        metadata={
            "name": "AD_PREV_ID",
            "type": "Element",
            "namespace": "",
        },
    )
    ad_method: None | AdMethodType = field(
        default=None,
        metadata={
            "name": "AD_METHOD",
            "type": "Element",
            "namespace": "",
        },
    )
    attitude_source: None | str = field(
        default=None,
        metadata={
            "name": "ATTITUDE_SOURCE",
            "type": "Element",
            "namespace": "",
        },
    )
    number_states: None | int = field(
        default=None,
        metadata={
            "name": "NUMBER_STATES",
            "type": "Element",
            "namespace": "",
        },
    )
    attitude_states: AcmAttitudeType = field(
        metadata={
            "name": "ATTITUDE_STATES",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    euler_rot_seq: None | RotseqType = field(
        default=None,
        metadata={
            "name": "EULER_ROT_SEQ",
            "type": "Element",
            "namespace": "",
        },
    )
    cov_type: None | AcmCovarianceLineType = field(
        default=None,
        metadata={
            "name": "COV_TYPE",
            "type": "Element",
            "namespace": "",
        },
    )
    ref_frame_a: str = field(
        metadata={
            "name": "REF_FRAME_A",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    ref_frame_b: str = field(
        metadata={
            "name": "REF_FRAME_B",
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    rate_states: None | AttRateType = field(
        default=None,
        metadata={
            "name": "RATE_STATES",
            "type": "Element",
            "namespace": "",
        },
    )
    sigma_u: None | SigmaUtype = field(
        default=None,
        metadata={
            "name": "SIGMA_U",
            "type": "Element",
            "namespace": "",
        },
    )
    sigma_v: None | SigmaVtype = field(
        default=None,
        metadata={
            "name": "SIGMA_V",
            "type": "Element",
            "namespace": "",
        },
    )
    rate_process_noise_stddev: None | SigmaUtype = field(
        default=None,
        metadata={
            "name": "RATE_PROCESS_NOISE_STDDEV",
            "type": "Element",
            "namespace": "",
        },
    )
    sensor_data: list[SensorDataType] = field(
        default_factory=list,
        metadata={
            "name": "sensorData",
            "type": "Element",
            "namespace": "",
        },
    )


@dataclass(kw_only=True)
class AcmData:
    class Meta:
        name = "acmData"

    att: list[AcmAttitudeStateType] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "",
        },
    )
    phys: None | AcmPhysicalDescriptionType = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "",
        },
    )
    cov: list[AcmCovarianceMatrixType] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "",
        },
    )
    man: list[AcmManeuverParametersType] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "",
        },
    )
    ad: None | AcmAdParametersType = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "",
        },
    )
    user: None | UserDefinedType = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "",
        },
    )


@dataclass(kw_only=True)
class AcmSegment:
    class Meta:
        name = "acmSegment"

    metadata: AcmMetadata = field(
        metadata={
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    data: AcmData = field(
        metadata={
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class AcmBody:
    class Meta:
        name = "acmBody"

    segment: AcmSegment = field(
        metadata={
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )


@dataclass(kw_only=True)
class AcmType:
    class Meta:
        name = "acmType"

    header: AdmHeader = field(
        metadata={
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    body: AcmBody = field(
        metadata={
            "type": "Element",
            "namespace": "",
            "required": True,
        }
    )
    id: str = field(
        init=False,
        default="CCSDS_ACM_VERS",
        metadata={
            "type": "Attribute",
            "required": True,
        },
    )
    version: str = field(
        init=False,
        default="2.0",
        metadata={
            "type": "Attribute",
            "required": True,
        },
    )



# CCSDS-NDM: CCSDS Navigation Data Messages Read/Write Library
#
# Copyright (C) Egemen Imre
#
# Licensed under GNU GPL v3.0. See LICENSE.rst for more info.
"""
Tests for the NDM File I/O Operations for KVN.

"""

from pathlib import Path

import pytest

from ccsds_ndm.mapping import NDMFileFormats
from ccsds_ndm.ndm_io import NdmIo
from tests.test_ndm_io import extra_path, process_paths

file_paths = {
    "OPM_1": Path("data", "kvn", "502x0b2c1e2_fig3_2_opm"),
    "OPM_2": Path("data", "kvn", "502x0b2c1e2_fig3_4_opm"),
    "OEM_1": Path("data", "kvn", "502x0b2c1e2_fig5_1_oem"),
    "OEM_2": Path("data", "kvn", "odmv2-testcase6_abbrev"),
    "OEM_3": Path("data", "kvn", "odmv2-testcase7a_xxx"),
    "APM_1": Path("data", "kvn", "504x0b1c1_fig3_6_apm"),
    "APM_2": Path("data", "kvn", "504x0b1c1_fig3_8_apm"),
    "RDM_1": Path("data", "kvn", "508x1b1_figc_2_rdm"),
    "ADM_1": Path("data", "kvn", "adm-testcase04a_abbrev"),
    "ADM_2": Path("data", "kvn", "adm-testcase04a_multi"),
    "CDM_1": Path("data", "kvn", "cdm_example_section4"),
    "OMM_1": Path("data", "kvn", "omm1_ct"),
    "OMM_2": Path("data", "kvn", "omm1_st"),
    "TDM_1": Path("data", "kvn", "tdm-testcase01b"),
}


# def test_read_file():

#     path = Path("data", "kvn", "504x0b1c1_fig3_8_apm.kvn")
#     # path = Path("data", "kvn", "adm-testcase04a_multi.kvn")

#     # *** read KVN files ***
#     working_dir = Path.cwd()

#     if path is not None:
#         kvn_path = process_paths(working_dir, path)

#         # read KVN file
#         ndm = NdmKvnIo().from_path(kvn_path)

# print(ndm)
# print(NdmKvnIo().to_string(ndm, NDMFileFormats.XML))


def test_write_multi_ndm_kvn():
    """Combi-NDM for KVN fail test."""
    with pytest.raises(NotImplementedError):
        # this should throw an NotImplementedError exception

        ndm_v2 = Path("data", "xml", "omm_combined.xml")

        # check path and correct if necessary
        kvn_path = Path.cwd().joinpath(ndm_v2)
        if not Path.cwd().joinpath(kvn_path).exists():
            kvn_path = Path.cwd().joinpath(extra_path).joinpath(ndm_v2)

        ndm = NdmIo().from_path(kvn_path)
        NdmIo().to_file(ndm, NDMFileFormats.KVN, Path("new.kvn"))


@pytest.mark.parametrize("ndm_key, path", file_paths.items())
def test_round_trip(ndm_key, path: Path):
    """Tests a full KVN round-trip.

    Reads the KVN file, exports to a KVN string, re-parses that string, exports
    it back to KVN again, and compares the two KVN strings."""

    working_dir = Path.cwd()

    if path is not None:
        kvn_path = process_paths(working_dir, path.with_suffix(".kvn"))

        ndm = NdmIo().from_path(kvn_path)
        kvn_str_1 = NdmIo().to_string(ndm, NDMFileFormats.KVN)

        ndm_reread = NdmIo().from_string(kvn_str_1)
        kvn_str_2 = NdmIo().to_string(ndm_reread, NDMFileFormats.KVN)

        assert (
            kvn_str_1 == kvn_str_2
        ), f"[{ndm_key}] KVN round-trip produced different output"


@pytest.mark.parametrize("ndm_key, path", file_paths.items())
def test_write_files(ndm_key, path: Path):
    """Tests that the KVN writer produces the same keywords as the original file.

    Reads the KVN file, builds the object, writes it back to a KVN string,
    and compares the ordered list of keyword names. Values are intentionally
    excluded because float formatting may differ (e.g. '.83483E-4' vs
    '8.3483e-05') while remaining semantically identical."""

    working_dir = Path.cwd()

    if path is not None:
        kvn_path = process_paths(working_dir, path.with_suffix(".kvn"))

        ndm = NdmIo().from_path(kvn_path)
        kvn_out = NdmIo().to_string(ndm, NDMFileFormats.KVN)

        original = kvn_path.read_text()

        def extract_kv_keys(text):
            """Return sorted list of keyword names from KV lines (those containing '=').

            Packed data lines and section markers are ignored — their content
            is covered by test_compare_objects."""
            return sorted(
                stripped.split("=")[0].strip()
                for line in text.splitlines()
                if (stripped := line.strip())
                and "=" in stripped
                and not stripped.startswith("COMMENT")
            )

        assert extract_kv_keys(kvn_out) == extract_kv_keys(
            original
        ), f"[{ndm_key}] KVN writer produced different keywords than the original file"
